@@ -1004,10 +1004,11 @@ _estimate_nports(int nclients, int cli_per_port)
 static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 {
 	int sock = -1;
-	short port = -1;
+	uint16_t port;
 	eio_obj_t *obj;
 	int i, rc = SLURM_SUCCESS;
 	pthread_attr_t attr;
+	uint16_t *ports;
 
 	debug("Entering _msg_thr_create()");
 	slurm_uid = (uid_t) slurm_get_slurm_user_id();
@@ -1022,8 +1023,15 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 	if (!message_socket_ops.timeout)
 		message_socket_ops.timeout = slurm_get_msg_timeout() * 8000;
 
+	ports = slurm_get_srun_port_range();
 	for (i = 0; i < sls->num_resp_port; i++) {
-		if (net_stream_listen(&sock, &port) < 0) {
+		int cc;
+
+		if (ports)
+			cc = net_stream_listen_ports(&sock, &port, ports);
+		else
+			cc = net_stream_listen(&sock, &port);
+		if (cc < 0) {
 			error("unable to initialize step launch listening "
 			      "socket: %m");
 			return SLURM_ERROR;
