@@ -46,6 +46,11 @@
 #include "src/common/slurmdbd_defs.h"
 #include "src/common/env.h"
 
+#define SLURMDBD_2_5_VERSION   11	/* slurm version 2.5 */
+
+#define MAX_PURGE_LIMIT 50000 /* Number of records that are purged at a time
+				 so that locks can be periodically released. */
+
 typedef struct {
 	char *cluster_nodes;
 	char *cpu_count;
@@ -62,6 +67,9 @@ typedef struct {
 	char *alloc_cpus;
 	char *alloc_nodes;
 	char *associd;
+	char *array_jobid;
+	char *array_max_tasks;
+	char *array_taskid;
 	char *blockid;
 	char *derived_ec;
 	char *derived_es;
@@ -189,9 +197,12 @@ enum {
  * enum below */
 static char *job_req_inx[] = {
 	"account",
+	"array_max_tasks",
 	"cpus_alloc",
 	"nodes_alloc",
 	"id_assoc",
+	"id_array_job",
+	"id_array_task",
 	"id_block",
 	"derived_ec",
 	"derived_es",
@@ -224,9 +235,12 @@ static char *job_req_inx[] = {
 
 enum {
 	JOB_REQ_ACCOUNT,
+	JOB_REQ_ARRAY_MAX,
 	JOB_REQ_ALLOC_CPUS,
 	JOB_REQ_ALLOC_NODES,
 	JOB_REQ_ASSOCID,
+	JOB_REQ_ARRAYJOBID,
+	JOB_REQ_ARRAYTASKID,
 	JOB_REQ_BLOCKID,
 	JOB_REQ_DERIVED_EC,
 	JOB_REQ_DERIVED_ES,
@@ -444,6 +458,9 @@ static void _pack_local_job(local_job_t *object,
 	packstr(object->alloc_cpus, buffer);
 	packstr(object->alloc_nodes, buffer);
 	packstr(object->associd, buffer);
+	packstr(object->array_jobid, buffer);
+	packstr(object->array_max_tasks, buffer);
+	packstr(object->array_taskid, buffer);
 	packstr(object->blockid, buffer);
 	packstr(object->derived_ec, buffer);
 	packstr(object->derived_es, buffer);
@@ -458,12 +475,12 @@ static void _pack_local_job(local_job_t *object,
 	packstr(object->name, buffer);
 	packstr(object->nodelist, buffer);
 	packstr(object->node_inx, buffer);
-	packstr(object->partition, buffer);
-	packstr(object->priority, buffer);
-	packstr(object->qos, buffer);
-	packstr(object->req_cpus, buffer);
-	packstr(object->req_mem, buffer);
-	packstr(object->resvid, buffer);
+	packstr(object->partition, buffer); /* priority */
+	packstr(object->priority, buffer);  /* qos */
+	packstr(object->qos, buffer);       /* req_cpus */
+	packstr(object->req_cpus, buffer);  /* req_mem */
+	packstr(object->req_mem, buffer);   /* resvid */
+	packstr(object->resvid, buffer);    /* partition */
 	packstr(object->start, buffer);
 	packstr(object->state, buffer);
 	packstr(object->submit, buffer);
@@ -481,7 +498,58 @@ static int _unpack_local_job(local_job_t *object,
 {
 	uint32_t tmp32;
 
-	if (rpc_version >= SLURMDBD_2_6_VERSION) {
+	/* For protocols <= 14_11, job_req_inx and it's corresponding enum,
+	 * were out of sync. This caused the following variables to have the
+	 * corresponding values:
+	 * job->partition = priority
+	 * job->priority  = qos
+	 * job->qos       = req_cpus
+	 * job->req_cpus  = req_mem
+	 * job->req_mem   = resvid
+	 * job->resvid    = partition
+	 *
+	 * The values were packed in the above order. To unpack the values
+	 * into the correct variables, the unpacking order is changed to
+	 * accomodate the shift in values. job->partition is unpacked before
+	 * job->start instead of after job->node_inx. */
+
+	if (rpc_version >= SLURM_14_11_PROTOCOL_VERSION) {
+		unpackstr_ptr(&object->account, &tmp32, buffer);
+		unpackstr_ptr(&object->alloc_cpus, &tmp32, buffer);
+		unpackstr_ptr(&object->alloc_nodes, &tmp32, buffer);
+		unpackstr_ptr(&object->associd, &tmp32, buffer);
+		unpackstr_ptr(&object->array_jobid, &tmp32, buffer);
+		unpackstr_ptr(&object->array_max_tasks, &tmp32, buffer);
+		unpackstr_ptr(&object->array_taskid, &tmp32, buffer);
+		unpackstr_ptr(&object->blockid, &tmp32, buffer);
+		unpackstr_ptr(&object->derived_ec, &tmp32, buffer);
+		unpackstr_ptr(&object->derived_es, &tmp32, buffer);
+		unpackstr_ptr(&object->exit_code, &tmp32, buffer);
+		unpackstr_ptr(&object->timelimit, &tmp32, buffer);
+		unpackstr_ptr(&object->eligible, &tmp32, buffer);
+		unpackstr_ptr(&object->end, &tmp32, buffer);
+		unpackstr_ptr(&object->gid, &tmp32, buffer);
+		unpackstr_ptr(&object->id, &tmp32, buffer);
+		unpackstr_ptr(&object->jobid, &tmp32, buffer);
+		unpackstr_ptr(&object->kill_requid, &tmp32, buffer);
+		unpackstr_ptr(&object->name, &tmp32, buffer);
+		unpackstr_ptr(&object->nodelist, &tmp32, buffer);
+		unpackstr_ptr(&object->node_inx, &tmp32, buffer);
+		unpackstr_ptr(&object->priority, &tmp32, buffer);
+		unpackstr_ptr(&object->qos, &tmp32, buffer);
+		unpackstr_ptr(&object->req_cpus, &tmp32, buffer);
+		unpackstr_ptr(&object->req_mem, &tmp32, buffer);
+		unpackstr_ptr(&object->resvid, &tmp32, buffer);
+		unpackstr_ptr(&object->partition, &tmp32, buffer);
+		unpackstr_ptr(&object->start, &tmp32, buffer);
+		unpackstr_ptr(&object->state, &tmp32, buffer);
+		unpackstr_ptr(&object->submit, &tmp32, buffer);
+		unpackstr_ptr(&object->suspended, &tmp32, buffer);
+		unpackstr_ptr(&object->track_steps, &tmp32, buffer);
+		unpackstr_ptr(&object->uid, &tmp32, buffer);
+		unpackstr_ptr(&object->wckey, &tmp32, buffer);
+		unpackstr_ptr(&object->wckey_id, &tmp32, buffer);
+	} else if (rpc_version >= SLURMDBD_2_6_VERSION) {
 		unpackstr_ptr(&object->account, &tmp32, buffer);
 		unpackstr_ptr(&object->alloc_cpus, &tmp32, buffer);
 		unpackstr_ptr(&object->alloc_nodes, &tmp32, buffer);
@@ -500,12 +568,12 @@ static int _unpack_local_job(local_job_t *object,
 		unpackstr_ptr(&object->name, &tmp32, buffer);
 		unpackstr_ptr(&object->nodelist, &tmp32, buffer);
 		unpackstr_ptr(&object->node_inx, &tmp32, buffer);
-		unpackstr_ptr(&object->partition, &tmp32, buffer);
 		unpackstr_ptr(&object->priority, &tmp32, buffer);
 		unpackstr_ptr(&object->qos, &tmp32, buffer);
 		unpackstr_ptr(&object->req_cpus, &tmp32, buffer);
 		unpackstr_ptr(&object->req_mem, &tmp32, buffer);
 		unpackstr_ptr(&object->resvid, &tmp32, buffer);
+		unpackstr_ptr(&object->partition, &tmp32, buffer);
 		unpackstr_ptr(&object->start, &tmp32, buffer);
 		unpackstr_ptr(&object->state, &tmp32, buffer);
 		unpackstr_ptr(&object->submit, &tmp32, buffer);
@@ -533,11 +601,11 @@ static int _unpack_local_job(local_job_t *object,
 		unpackstr_ptr(&object->name, &tmp32, buffer);
 		unpackstr_ptr(&object->nodelist, &tmp32, buffer);
 		unpackstr_ptr(&object->node_inx, &tmp32, buffer);
-		unpackstr_ptr(&object->partition, &tmp32, buffer);
 		unpackstr_ptr(&object->priority, &tmp32, buffer);
 		unpackstr_ptr(&object->qos, &tmp32, buffer);
 		unpackstr_ptr(&object->req_cpus, &tmp32, buffer);
 		unpackstr_ptr(&object->resvid, &tmp32, buffer);
+		unpackstr_ptr(&object->partition, &tmp32, buffer);
 		unpackstr_ptr(&object->start, &tmp32, buffer);
 		unpackstr_ptr(&object->state, &tmp32, buffer);
 		unpackstr_ptr(&object->submit, &tmp32, buffer);
@@ -672,7 +740,7 @@ static void _pack_local_step(local_step_t *object,
 		packstr(object->task_dist, buffer);
 		packstr(object->user_sec, buffer);
 		packstr(object->user_usec, buffer);
-	} else if (rpc_version >= SLURMDBD_VERSION_MIN) {
+	} else {
 		packstr(object->ave_cpu, buffer);
 		packstr(object->ave_pages, buffer);
 		packstr(object->ave_rss, buffer);
@@ -803,7 +871,7 @@ static int _unpack_local_step(local_step_t *object,
 		unpackstr_ptr(&object->task_dist, &tmp32, buffer);
 		unpackstr_ptr(&object->user_sec, &tmp32, buffer);
 		unpackstr_ptr(&object->user_usec, &tmp32, buffer);
-	} else if (rpc_version >= SLURMDBD_VERSION_MIN) {
+	} else {
 		unpackstr_ptr(&object->ave_cpu, &tmp32, buffer);
 		unpackstr_ptr(&object->ave_pages, &tmp32, buffer);
 		unpackstr_ptr(&object->ave_rss, &tmp32, buffer);
@@ -1407,8 +1475,8 @@ static uint32_t _archive_events(mysql_conn_t *mysql_conn, char *cluster_name,
 	xfree(tmp);
 
 //	START_TIMER;
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return SLURM_ERROR;
@@ -1536,8 +1604,8 @@ static uint32_t _archive_jobs(mysql_conn_t *mysql_conn, char *cluster_name,
 	xfree(tmp);
 
 //	START_TIMER;
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return SLURM_ERROR;
@@ -1566,6 +1634,9 @@ static uint32_t _archive_jobs(mysql_conn_t *mysql_conn, char *cluster_name,
 		job.alloc_cpus = row[JOB_REQ_ALLOC_CPUS];
 		job.alloc_nodes = row[JOB_REQ_ALLOC_NODES];
 		job.associd = row[JOB_REQ_ASSOCID];
+		job.array_jobid = row[JOB_REQ_ARRAYJOBID];
+		job.array_max_tasks = row[JOB_REQ_ARRAY_MAX];
+		job.array_taskid = row[JOB_REQ_ARRAYTASKID];
 		job.blockid = row[JOB_REQ_BLOCKID];
 		job.derived_ec = row[JOB_REQ_DERIVED_EC];
 		job.derived_es = row[JOB_REQ_DERIVED_ES];
@@ -1580,12 +1651,12 @@ static uint32_t _archive_jobs(mysql_conn_t *mysql_conn, char *cluster_name,
 		job.name = row[JOB_REQ_NAME];
 		job.nodelist = row[JOB_REQ_NODELIST];
 		job.node_inx = row[JOB_REQ_NODE_INX];
-		job.partition = row[JOB_REQ_PARTITION];
-		job.priority = row[JOB_REQ_PRIORITY];
-		job.qos = row[JOB_REQ_QOS];
-		job.req_cpus = row[JOB_REQ_REQ_CPUS];
-		job.req_mem = row[JOB_REQ_REQ_MEM];
-		job.resvid = row[JOB_REQ_RESVID];
+		job.partition = row[JOB_REQ_PARTITION]; /* priority */
+		job.priority = row[JOB_REQ_PRIORITY];   /* qos */
+		job.qos = row[JOB_REQ_QOS];             /* cpus_req */
+		job.req_cpus = row[JOB_REQ_REQ_CPUS];   /* mem_req */
+		job.req_mem = row[JOB_REQ_REQ_MEM];     /* id_resv */
+		job.resvid = row[JOB_REQ_RESVID];       /* partition */
 		job.start = row[JOB_REQ_START];
 		job.state = row[JOB_REQ_STATE];
 		job.submit = row[JOB_REQ_SUBMIT];
@@ -1644,9 +1715,12 @@ static char *_load_jobs(uint16_t rpc_version, Buf buffer,
 
 		xstrfmtcat(insert, format,
 			   object.account,
+			   object.array_max_tasks,
 			   object.alloc_cpus,
 			   object.alloc_nodes,
 			   object.associd,
+			   object.array_jobid,
+			   object.array_taskid,
 			   object.blockid,
 			   object.derived_ec,
 			   object.derived_es,
@@ -1712,8 +1786,8 @@ static uint32_t _archive_resvs(mysql_conn_t *mysql_conn, char *cluster_name,
 	xfree(tmp);
 
 //	START_TIMER;
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return SLURM_ERROR;
@@ -1841,8 +1915,8 @@ static uint32_t _archive_steps(mysql_conn_t *mysql_conn, char *cluster_name,
 	xfree(tmp);
 
 //	START_TIMER;
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return SLURM_ERROR;
@@ -2045,8 +2119,8 @@ static uint32_t _archive_suspend(mysql_conn_t *mysql_conn, char *cluster_name,
 	xfree(tmp);
 
 //	START_TIMER;
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return SLURM_ERROR;
@@ -2176,11 +2250,16 @@ static int _execute_archive(mysql_conn_t *mysql_conn,
 				return rc;
 		}
 		query = xstrdup_printf("delete from \"%s_%s\" where "
-				       "time_start <= %ld && time_end != 0",
-				       cluster_name, event_table, curr_end);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
-		rc = mysql_db_query(mysql_conn, query);
+				       "time_start <= %ld && time_end != 0 "
+				       "LIMIT %d",
+				       cluster_name, event_table, curr_end,
+				       MAX_PURGE_LIMIT);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+
+		while ((rc = mysql_db_delete_affected_rows(
+						mysql_conn, query)) > 0);
+
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
 			error("Couldn't remove old event data");
@@ -2213,11 +2292,15 @@ exit_events:
 				return rc;
 		}
 		query = xstrdup_printf("delete from \"%s_%s\" where "
-				       "time_start <= %ld && time_end != 0",
-				       cluster_name, suspend_table, curr_end);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
-		rc = mysql_db_query(mysql_conn, query);
+				       "time_start <= %ld && time_end != 0 "
+				       "LIMIT %d",
+				       cluster_name, suspend_table, curr_end,
+				       MAX_PURGE_LIMIT);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+
+		while ((rc = mysql_db_delete_affected_rows(
+						mysql_conn, query)) > 0);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
 			error("Couldn't remove old suspend data");
@@ -2251,11 +2334,16 @@ exit_suspend:
 		}
 
 		query = xstrdup_printf("delete from \"%s_%s\" where "
-				       "time_start <= %ld && time_end != 0",
-				       cluster_name, step_table, curr_end);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
-		rc = mysql_db_query(mysql_conn, query);
+				       "time_start <= %ld && time_end != 0 "
+				       "LIMIT %d",
+				       cluster_name, step_table, curr_end,
+				       MAX_PURGE_LIMIT);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+
+		while ((rc = mysql_db_delete_affected_rows(
+						mysql_conn, query)) > 0);
+
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
 			error("Couldn't remove old step data");
@@ -2289,11 +2377,15 @@ exit_steps:
 
 		query = xstrdup_printf("delete from \"%s_%s\" "
 				       "where time_submit <= %ld "
-				       "&& time_end != 0",
-				       cluster_name, job_table, curr_end);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
-		rc = mysql_db_query(mysql_conn, query);
+				       "&& time_end != 0 LIMIT %d",
+				       cluster_name, job_table, curr_end,
+				       MAX_PURGE_LIMIT);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+
+		while ((rc = mysql_db_delete_affected_rows(
+						mysql_conn, query)) > 0);
+
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
 			error("Couldn't remove old job data");
@@ -2317,8 +2409,8 @@ exit_jobs:
 
 		if (SLURMDB_PURGE_ARCHIVE_SET(arch_cond->purge_resv)) {
 			rc = _archive_resvs(mysql_conn, cluster_name,
-					   curr_end, arch_cond->archive_dir,
-					   arch_cond->purge_resv);
+					    curr_end, arch_cond->archive_dir,
+					    arch_cond->purge_resv);
 			if (!rc)
 				goto exit_resvs;
 			else if (rc == SLURM_ERROR)
@@ -2327,11 +2419,15 @@ exit_jobs:
 
 		query = xstrdup_printf("delete from \"%s_%s\" "
 				       "where time_start <= %ld "
-				       "&& time_end != 0",
-				       cluster_name, resv_table, curr_end);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
-		rc = mysql_db_query(mysql_conn, query);
+				       "&& time_end != 0 LIMIT %d",
+				       cluster_name, resv_table, curr_end,
+				       MAX_PURGE_LIMIT);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
+
+		while ((rc = mysql_db_delete_affected_rows(
+						mysql_conn, query)) > 0);
+
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
 			error("Couldn't remove old resv data");
@@ -2451,12 +2547,19 @@ extern int as_mysql_jobacct_process_archive_load(
 	buffer = create_buf(data, data_size);
 
 	safe_unpack16(&ver, buffer);
-	debug3("Version in assoc_mgr_state header is %u", ver);
-	if (ver <= SLURM_PROTOCOL_VERSION || ver < SLURMDBD_VERSION_MIN) {
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn,
+			 "Version in assoc_mgr_state header is %u", ver);
+	/* Don't verify the lower limit as we should be keeping all
+	   older versions around here just to support super old
+	   archive files since they don't get regenerated all the
+	   time.
+	*/
+	if (ver > SLURM_PROTOCOL_VERSION) {
 		error("***********************************************");
 		error("Can not recover archive file, incompatible version, "
-		      "got %u need > %u <= %u", ver,
-		      SLURMDBD_VERSION_MIN, SLURM_PROTOCOL_VERSION);
+		      "got %u need <= %u", ver,
+		      SLURM_PROTOCOL_VERSION);
 		error("***********************************************");
 		free_buf(buffer);
 		return EFAULT;
@@ -2500,8 +2603,8 @@ got_sql:
 		error("No data to load");
 		return SLURM_ERROR;
 	}
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, data);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", data);
 	error_code = mysql_db_query_check_after(mysql_conn, data);
 	xfree(data);
 	if (error_code != SLURM_SUCCESS) {
