@@ -228,13 +228,13 @@ static int _process_cluster_usage(mysql_conn_t *mysql_conn,
 
 		total_used = c_usage->a_cpu +
 			c_usage->d_cpu + c_usage->pd_cpu;
-		/* info("We now have (%"PRIu64"+%"PRIu64"+" */
-		/*      "%"PRIu64")(%"PRIu64") " */
-		/*       "?= %"PRIu64"", */
-		/*       c_usage->a_cpu, c_usage->d_cpu, */
-		/*       c_usage->pd_cpu, total_used, */
-		/*       c_usage->total_time); */
 	}
+	/* info("Cluster %s now has (%"PRIu64"+%"PRIu64"+" */
+	/*      "%"PRIu64")(%"PRIu64") ?= %"PRIu64"", */
+	/*      cluster_name, */
+	/*      c_usage->a_cpu, c_usage->d_cpu, */
+	/*      c_usage->pd_cpu, total_used, */
+	/*      c_usage->total_time); */
 
 	c_usage->i_cpu = c_usage->total_time - total_used - c_usage->r_cpu;
 	/* sanity check just to make sure we have a
@@ -301,8 +301,8 @@ static int _process_cluster_usage(mysql_conn_t *mysql_conn,
 			   "resv_cpu_secs=VALUES(resv_cpu_secs), "
 			   "consumed_energy=VALUES(consumed_energy)",
 			   now);
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS)
@@ -323,6 +323,8 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 	MYSQL_RES *result = NULL;
 	MYSQL_ROW row;
 	int i = 0;
+	ListIterator c_itr = NULL;
+	local_cluster_usage_t *loc_c_usage;
 
 	char *event_req_inx[] = {
 		"node_name",
@@ -360,14 +362,14 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 			       curr_end, curr_start);
 	xfree(event_str);
 
-	debug3("%d(%s:%d) query\n%s",
-	       mysql_conn->conn, THIS_FILE, __LINE__, query);
+	if (debug_flags & DEBUG_FLAG_DB_USAGE)
+		DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 	if (!(result = mysql_db_query_ret(mysql_conn, query, 0))) {
 		xfree(query);
 		return NULL;
 	}
 	xfree(query);
-
+	c_itr = list_iterator_create(cluster_down_list);
 	while ((row = mysql_fetch_row(result))) {
 		time_t row_start = slurm_atoul(row[EVENT_REQ_START]);
 		time_t row_end = slurm_atoul(row[EVENT_REQ_END]);
@@ -395,8 +397,6 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 			 * for the entire period.
 			 */
 			if (state || !c_usage) {
-				local_cluster_usage_t *loc_c_usage;
-
 				loc_c_usage = xmalloc(
 					sizeof(local_cluster_usage_t));
 				loc_c_usage->cpu_count = row_cpu;
@@ -431,8 +431,8 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 		   time period we would already have it.
 		*/
 		if (c_usage) {
-			int local_start = row_start;
-			int local_end = row_end;
+			time_t local_start = row_start;
+			time_t local_end = row_end;
 			int seconds;
 			if (c_usage->start > local_start)
 				local_start = c_usage->start;
@@ -440,20 +440,56 @@ static local_cluster_usage_t *_setup_cluster_usage(mysql_conn_t *mysql_conn,
 				local_end = c_usage->end;
 			seconds = (local_end - local_start);
 			if (seconds > 0) {
-				/* info("node %s adds " */
-				/*      "(%d)(%d-%d) * %d = %d " */
-				/*      "to %d", */
+				/* info("%p node %s adds " */
+				/*      "(%d)(%ld-%ld) * %d = %"PRIu64" " */
+				/*      "to %"PRIu64" (%s - %s)", */
+				/*      c_usage, */
 				/*      row[EVENT_REQ_NAME], */
 				/*      seconds, */
 				/*      local_end, local_start, */
 				/*      row_cpu, */
-				/*      seconds * row_cpu, */
-				/*      row_cpu); */
-				c_usage->d_cpu += seconds * row_cpu;
+				/*      seconds * (uint64_t)row_cpu, */
+				/*      c_usage->d_cpu, */
+				/*      slurm_ctime(&local_start), */
+				/*      slurm_ctime(&local_end)); */
+				c_usage->d_cpu += seconds * (uint64_t)row_cpu;
+				/* Now remove this time if there was a
+				   disconnected slurmctld during the
+				   down time.
+				*/
+				list_iterator_reset(c_itr);
+				while ((loc_c_usage = list_next(c_itr))) {
+					int temp_end = row_end;
+					int temp_start = row_start;
+					if (loc_c_usage->start > local_start)
+						temp_start = loc_c_usage->start;
+					if (loc_c_usage->end < temp_end)
+						temp_end = loc_c_usage->end;
+					seconds = (temp_end - temp_start);
+					if (seconds < 1)
+						continue;
+
+					seconds *= row_cpu;
+					if (seconds >= loc_c_usage->total_time)
+						loc_c_usage->total_time = 0;
+					else
+						loc_c_usage->total_time -=
+							seconds;
+
+					/* info("Node %s was down for " */
+					/*      "%d seconds while " */
+					/*      "cluster %s's slurmctld " */
+					/*      "wasn't responding %"PRIu64, */
+					/*      row[EVENT_REQ_NAME], */
+					/*      seconds, cluster_name, */
+					/*      loc_c_usage->total_time); */
+				}
 			}
 		}
 	}
 	mysql_free_result(result);
+	list_iterator_destroy(c_itr);
+
 	return c_usage;
 }
 
@@ -487,6 +523,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		"job.id_job",
 		"job.id_assoc",
 		"job.id_wckey",
+		"job.array_task_pending",
 		"job.time_eligible",
 		"job.time_start",
 		"job.time_end",
@@ -502,6 +539,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		JOB_REQ_JOBID,
 		JOB_REQ_ASSOCID,
 		JOB_REQ_WCKEYID,
+		JOB_REQ_ARRAY_PENDING,
 		JOB_REQ_ELG,
 		JOB_REQ_START,
 		JOB_REQ_END,
@@ -578,8 +616,10 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 		local_id_usage_t *a_usage = NULL;
 		local_id_usage_t *w_usage = NULL;
 
-		debug3("%s curr hour is now %ld-%ld",
-		       cluster_name, curr_start, curr_end);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn,
+				 "%s curr hour is now %ld-%ld",
+				 cluster_name, curr_start, curr_end);
 /* 		info("start %s", slurm_ctime(&curr_start)); */
 /* 		info("end %s", slurm_ctime(&curr_end)); */
 
@@ -602,10 +642,10 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				       curr_end, curr_start,
 				       RESERVE_FLAG_IGN_JOBS);
 
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 		if (!(result = mysql_db_query_ret(
-			     mysql_conn, query, 0))) {
+			      mysql_conn, query, 0))) {
 			xfree(query);
 			_destroy_local_cluster_usage(c_usage);
 			return SLURM_ERROR;
@@ -707,10 +747,10 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				       cluster_name, step_table,
 				       curr_end, curr_start);
 
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 		if (!(result = mysql_db_query_ret(
-			     mysql_conn, query, 0))) {
+			      mysql_conn, query, 0))) {
 			xfree(query);
 			_destroy_local_cluster_usage(c_usage);
 			return SLURM_ERROR;
@@ -721,6 +761,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 			uint32_t job_id = slurm_atoul(row[JOB_REQ_JOBID]);
 			uint32_t assoc_id = slurm_atoul(row[JOB_REQ_ASSOCID]);
 			uint32_t wckey_id = slurm_atoul(row[JOB_REQ_WCKEYID]);
+			uint32_t array_pending =
+				slurm_atoul(row[JOB_REQ_ARRAY_PENDING]);
 			uint32_t resv_id = slurm_atoul(row[JOB_REQ_RESVID]);
 			time_t row_eligible = slurm_atoul(row[JOB_REQ_ELG]);
 			time_t row_start = slurm_atoul(row[JOB_REQ_START]);
@@ -765,8 +807,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				       mysql_conn->conn, THIS_FILE,
 				       __LINE__, query);
 				if (!(result2 = mysql_db_query_ret(
-					     mysql_conn,
-					     query, 0))) {
+					      mysql_conn,
+					      query, 0))) {
 					xfree(query);
 					_destroy_local_cluster_usage(c_usage);
 					return SLURM_ERROR;
@@ -846,15 +888,18 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				if (loc_c_usage->end < temp_end)
 					temp_end = loc_c_usage->end;
 				loc_seconds = (temp_end - temp_start);
-				if (loc_seconds > 0) {
-					/* info(" Job %u was running for " */
-					/*      "%"PRIu64" seconds while " */
-					/*      "cluster %s's slurmctld " */
-					/*      "wasn't responding", */
-					/*      job_id, */
-					/*      (uint64_t) */
-					/*      (seconds * row_acpu), */
-					/*      cluster_name); */
+				if (loc_seconds < 1)
+					continue;
+
+				loc_seconds *= row_acpu;
+				/* info(" Job %u was running for " */
+				/*      "%d seconds while " */
+				/*      "cluster %s's slurmctld " */
+				/*      "wasn't responding", */
+				/*      job_id, loc_seconds, cluster_name); */
+				if (loc_seconds >= loc_c_usage->total_time)
+					loc_c_usage->total_time = 0;
+				else {
 					loc_c_usage->total_time -=
 						loc_seconds * row_acpu;
 				}
@@ -897,7 +942,7 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 							temp_end = r_usage->end;
 
 						if ((temp_end - temp_start)
-						   > 0) {
+						    > 0) {
 							r_usage->a_cpu +=
 								(temp_end
 								 - temp_start)
@@ -941,16 +986,27 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 					temp_end = c_usage->end;
 				loc_seconds = (temp_end - temp_start);
 				if (loc_seconds > 0) {
+					/* If we have pending jobs in
+					   an array they haven't been
+					   inserted into the database
+					   yet as proper job records,
+					   so handle them here.
+					*/
+					if (array_pending)
+						loc_seconds *= array_pending;
+
 					/* info("%d assoc %d reserved " */
-					/*      "(%d)(%d-%d) * %d = %d " */
+					/*      "(%d)(%d-%d) * %d * %d = %d " */
 					/*      "to %d", */
 					/*      job_id, */
 					/*      assoc_id, */
-					/*      seconds, */
+					/*      temp_end - temp_start, */
 					/*      temp_end, temp_start, */
 					/*      row_rcpu, */
-					/*      seconds * row_rcpu, */
+					/*      array_pending, */
+					/*      loc_seconds, */
 					/*      row_rcpu); */
+
 					c_usage->r_cpu +=
 						loc_seconds * row_rcpu;
 				}
@@ -1053,8 +1109,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				   "consumed_energy=VALUES(consumed_energy);",
 				   now);
 
-			debug3("%d(%s:%d) query\n%s",
-			       mysql_conn->conn, THIS_FILE, __LINE__, query);
+			if (debug_flags & DEBUG_FLAG_DB_USAGE)
+				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 			if (rc != SLURM_SUCCESS) {
@@ -1102,8 +1158,8 @@ extern int as_mysql_hourly_rollup(mysql_conn_t *mysql_conn,
 				   "consumed_energy=VALUES(consumed_energy);",
 				   now);
 
-			debug3("%d(%s:%d) query\n%s",
-			       mysql_conn->conn, THIS_FILE, __LINE__, query);
+			if (debug_flags & DEBUG_FLAG_DB_USAGE)
+				DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 			rc = mysql_db_query(mysql_conn, query);
 			xfree(query);
 			if (rc != SLURM_SUCCESS) {
@@ -1175,7 +1231,10 @@ extern int as_mysql_daily_rollup(mysql_conn_t *mysql_conn,
 	curr_end = mktime(&start_tm);
 
 	while (curr_start < end) {
-		debug3("curr day is now %ld-%ld", curr_start, curr_end);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn,
+				 "curr day is now %ld-%ld",
+				 curr_start, curr_end);
 /* 		info("start %s", slurm_ctime(&curr_start)); */
 /* 		info("end %s", slurm_ctime(&curr_end)); */
 		query = xstrdup_printf(
@@ -1242,8 +1301,8 @@ extern int as_mysql_daily_rollup(mysql_conn_t *mysql_conn,
 				   cluster_name, wckey_hour_table,
 				   curr_end, curr_start, now);
 		}
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
@@ -1300,7 +1359,10 @@ extern int as_mysql_monthly_rollup(mysql_conn_t *mysql_conn,
 	curr_end = mktime(&start_tm);
 
 	while (curr_start < end) {
-		debug3("curr month is now %ld-%ld", curr_start, curr_end);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn,
+				 "curr month is now %ld-%ld",
+				 curr_start, curr_end);
 /* 		info("start %s", slurm_ctime(&curr_start)); */
 /* 		info("end %s", slurm_ctime(&curr_end)); */
 		query = xstrdup_printf(
@@ -1368,8 +1430,8 @@ extern int as_mysql_monthly_rollup(mysql_conn_t *mysql_conn,
 				   cluster_name, wckey_day_table,
 				   curr_end, curr_start, now);
 		}
-		debug3("%d(%s:%d) query\n%s",
-		       mysql_conn->conn, THIS_FILE, __LINE__, query);
+		if (debug_flags & DEBUG_FLAG_DB_USAGE)
+			DB_DEBUG(mysql_conn->conn, "query\n%s", query);
 		rc = mysql_db_query(mysql_conn, query);
 		xfree(query);
 		if (rc != SLURM_SUCCESS) {
