@@ -229,6 +229,12 @@ extern int basil_inventory(void)
 	time_t now = time(NULL);
 	static time_t slurm_alps_mismatch_time = (time_t) 0;
 	static bool logged_sync_timeout = false;
+	static time_t last_inv_run = 0;
+
+	if ((now - last_inv_run) < inv_interval)
+		return SLURM_SUCCESS;
+
+	last_inv_run = now;
 
 	inv = get_full_inventory(version);
 	if (inv == NULL) {
@@ -748,7 +754,13 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 
 	if (cray_conf->sub_alloc) {
 		mppdepth = MAX(1, job_ptr->details->cpus_per_task);
-		mppnppn  = job_ptr->details->ntasks_per_node;
+		if (!job_ptr->details->ntasks_per_node
+		    && job_ptr->details->num_tasks) {
+			mppnppn = (job_ptr->details->num_tasks +
+				   job_ptr->job_resrcs->nhosts - 1) /
+				job_ptr->job_resrcs->nhosts;
+		} else
+			mppnppn  = job_ptr->details->ntasks_per_node;
 	} else {
 		/* always be 1 */
 		mppdepth = 1;
@@ -882,7 +894,7 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 		 * mppmem and use it as the level for all
 		 * nodes (mppmem is 0 when coming in).
 		 */
-		mppmem = min_memory / largest_cpus;
+		mppmem = MIN(min_memory, node_min_mem) / largest_cpus;
 
 		/* Minimum memory per processing element should be 1,
 		 * since 0 means give all the memory to the job. */
@@ -891,6 +903,7 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 	}
 
 	if (cray_conf->sub_alloc) {
+		int sock_core_inx = 0, sock_core_rep_cnt = 0;
 		mppwidth = 0;
 		/* mppwidth */
 		for (i = 0; i < job_ptr->job_resrcs->nhosts; i++) {
@@ -898,14 +911,22 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 			uint32_t node_tasks =
 				job_ptr->job_resrcs->cpus[i] / mppdepth;
 
-			if ((job_ptr->job_resrcs->sockets_per_node[i] > 0) &&
-			    (job_ptr->job_resrcs->cores_per_socket[i] > 0)) {
+			if ((job_ptr->job_resrcs->
+			     sockets_per_node[sock_core_inx] > 0) &&
+			    (job_ptr->job_resrcs->
+			     cores_per_socket[sock_core_inx] > 0)) {
 				hwthreads_per_core =
 					job_ptr->job_resrcs->cpus[i] /
 					job_ptr->job_resrcs->
-					sockets_per_node[i] /
+					sockets_per_node[sock_core_inx] /
 					job_ptr->job_resrcs->
-					cores_per_socket[i];
+					cores_per_socket[sock_core_inx];
+			}
+			if ((++sock_core_rep_cnt) > job_ptr->job_resrcs->
+			    sock_core_rep_count[sock_core_inx]) {
+				/* move to the next node */
+				sock_core_inx++;
+				sock_core_rep_cnt = 0;
 			}
 			if (nppcu)
 				node_tasks =
