@@ -151,8 +151,16 @@ static uint32_t blade_cnt = 0;
 static pthread_mutex_t blade_mutex = PTHREAD_MUTEX_INITIALIZER;
 static time_t last_npc_update;
 
-#ifdef HAVE_NATIVE_CRAY
+static int active_post_nhc_cnt = 0;
+static pthread_mutex_t throttle_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t throttle_cond = PTHREAD_COND_INITIALIZER;
 
+#if defined(HAVE_NATIVE_CRAY_GA) && !defined(HAVE_CRAY_NETWORK)
+static size_t topology_num_nodes = 0;
+static alpsc_topology_t *topology = NULL;
+#endif
+
+#ifdef HAVE_NATIVE_CRAY
 
 /* Used for aeld communication */
 alpsc_ev_app_t *app_list = NULL;	// List of running/suspended apps
@@ -168,6 +176,7 @@ pthread_mutex_t aeld_mutex = PTHREAD_MUTEX_INITIALIZER;	// Mutex for the above
 
 #define AELD_SESSION_INTERVAL	60	// aeld session retry interval (s)
 #define AELD_EVENT_INTERVAL	110	// aeld event sending interval (ms)
+#define AELD_LIST_CAPACITY	65536	// maximum list capacity
 
 /* Static functions used for aeld communication */
 static void _handle_aeld_error(const char *funcname, char *errmsg, int rv,
@@ -188,7 +197,7 @@ static void _update_app(struct job_record *job_ptr,
 			alpsc_ev_app_state_e state);
 #endif
 
-static uint32_t debug_flags = 0;
+static uint64_t debug_flags = 0;
 
 /*
  * These variables are required by the generic plugin interface.  If they
@@ -221,7 +230,7 @@ static uint32_t debug_flags = 0;
 const char plugin_name[]	= "Cray node selection plugin";
 const char plugin_type[]	= "select/cray";
 uint32_t plugin_id		= 107;
-const uint32_t plugin_version	= 100;
+const uint32_t plugin_version	= 120;
 
 extern int select_p_select_jobinfo_free(select_jobinfo_t *jobinfo);
 
@@ -320,6 +329,11 @@ fini:
 	xfree(nodelist_nids);
 
 	return status;
+
+#elif HAVE_CRAY_NETWORK
+	/* NHC not supported, but don't sleep before return. */
+	return 0;
+
 #else
 	if (debug_flags & DEBUG_FLAG_SELECT_TYPE)
 		info("simluating calling NHC for jobid %u "
@@ -402,9 +416,6 @@ static void _start_session(alpsc_ev_session_t **session, int *sessionfd)
 	while (1) {
 		pthread_mutex_lock(&aeld_mutex);
 
-		// Clear out the event list
-		_clear_event_list(event_list, &event_list_size);
-
 		// Create the session
 		rv = alpsc_ev_create_session(&errmsg, session, app_list,
 					     app_list_size);
@@ -526,7 +537,7 @@ static void _initialize_event(alpsc_ev_app_t *event,
 	event->nodes = NULL;
 	event->num_nodes = 0;
 
-	// Fill in nodes and num_nodes
+	// Fill in nodes and num_nodes if available
 	if (step_ptr->step_layout) {
 		hl = hostlist_create(step_ptr->step_layout->node_list);
 		if (hl == NULL) {
@@ -555,8 +566,6 @@ static void _initialize_event(alpsc_ev_app_t *event,
 
 		hostlist_iterator_destroy(hlit);
 		hostlist_destroy(hl);
-	} else {
-		// TODO: do we have to worry about batch scripts?
 	}
 	return;
 }
@@ -604,6 +613,9 @@ static void _add_to_app_list(alpsc_ev_app_t **list, int32_t *size,
 	if (*size + 1 > *capacity) {
 		if (*capacity == 0) {
 			*capacity = 16;
+		} else if (*capacity >= AELD_LIST_CAPACITY) {
+			debug("aeld list over capacity");
+			return;
 		} else {
 			*capacity *= 2;
 		}
@@ -638,10 +650,18 @@ static void _update_app(struct job_record *job_ptr,
 	// Fill in the new event
 	_initialize_event(&app, job_ptr, step_ptr, state);
 
+	// If there are no nodes, set_application_info will fail
+	if (app.nodes == NULL || app.num_nodes == 0) {
+		debug("Job %"PRIu32".%"PRIu32" has no nodes, skipping",
+		      job_ptr->job_id, step_ptr->step_id);
+		_free_event(&app);
+		return;
+	}
+
 	pthread_mutex_lock(&aeld_mutex);
 
 	// Add it to the event list, only if aeld is up
-	if (aeld_running == 2) {
+	if (aeld_running) {
 		_add_to_app_list(&event_list, &event_list_size,
 				 &event_list_capacity, &app);
 	}
@@ -766,17 +786,30 @@ static void _free_blade(blade_info_t *blade_info)
 static void _pack_blade(blade_info_t *blade_info, Buf buffer,
 			uint16_t protocol_version)
 {
-	pack64(blade_info->id, buffer);
-	pack32(blade_info->job_cnt, buffer);
-	pack_bit_str(blade_info->node_bitmap, buffer);
+	if (protocol_version >= SLURM_14_11_PROTOCOL_VERSION) {
+		pack64(blade_info->id, buffer);
+		pack32(blade_info->job_cnt, buffer);
+		pack_bit_str_hex(blade_info->node_bitmap, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		pack64(blade_info->id, buffer);
+		pack32(blade_info->job_cnt, buffer);
+		pack_bit_str(blade_info->node_bitmap, buffer);
+	}
+
 }
 
 static int _unpack_blade(blade_info_t *blade_info, Buf buffer,
 			 uint16_t protocol_version)
 {
-	safe_unpack64(&blade_info->id, buffer);
-	safe_unpack32(&blade_info->job_cnt, buffer);
-	unpack_bit_str(&blade_info->node_bitmap, buffer);
+	if (protocol_version >= SLURM_14_11_PROTOCOL_VERSION) {
+		safe_unpack64(&blade_info->id, buffer);
+		safe_unpack32(&blade_info->job_cnt, buffer);
+		unpack_bit_str_hex(&blade_info->node_bitmap, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		safe_unpack64(&blade_info->id, buffer);
+		safe_unpack32(&blade_info->job_cnt, buffer);
+		unpack_bit_str(&blade_info->node_bitmap, buffer);
+	}
 
 	return SLURM_SUCCESS;
 
@@ -843,6 +876,32 @@ static void _set_job_running_restore(select_jobinfo_t *jobinfo)
 		last_npc_update = time(NULL);
 }
 
+/* These functions prevent the fini's of jobs and steps from keeping
+ * the slurmctld write locks constantly set after the nhc is ran,
+ * which can prevent other RPCs and system functions from being
+ * processed. For example, a steady stream of step or job completions
+ * can prevent squeue from responding or jobs from being scheduled. */
+static void _throttle_start(void)
+{
+	slurm_mutex_lock(&throttle_mutex);
+	while (1) {
+		if (active_post_nhc_cnt == 0) {
+			active_post_nhc_cnt++;
+			break;
+		}
+		pthread_cond_wait(&throttle_cond, &throttle_mutex);
+	}
+	slurm_mutex_unlock(&throttle_mutex);
+	usleep(100);
+}
+static void _throttle_fini(void)
+{
+	slurm_mutex_lock(&throttle_mutex);
+	active_post_nhc_cnt--;
+	pthread_cond_broadcast(&throttle_cond);
+	slurm_mutex_unlock(&throttle_mutex);
+}
+
 static void *_job_fini(void *args)
 {
 	struct job_record *job_ptr = (struct job_record *)args;
@@ -874,6 +933,7 @@ static void *_job_fini(void *args)
 	/***********/
 	xfree(nhc_info.nodelist);
 
+	_throttle_start();
 	lock_slurmctld(job_write_lock);
 	if (job_ptr->magic == JOB_MAGIC) {
 		select_jobinfo_t *jobinfo = NULL;
@@ -889,6 +949,7 @@ static void *_job_fini(void *args)
 		      "this should never happen", nhc_info.jobid);
 
 	unlock_slurmctld(job_write_lock);
+	_throttle_fini();
 
 	return NULL;
 }
@@ -935,6 +996,7 @@ static void *_step_fini(void *args)
 
 	xfree(nhc_info.nodelist);
 
+	_throttle_start();
 	lock_slurmctld(job_write_lock);
 	if (!step_ptr->job_ptr) {
 		error("For some reason we don't have a job_ptr for "
@@ -966,6 +1028,7 @@ static void *_step_fini(void *args)
 		post_job_step(step_ptr);
 	}
 	unlock_slurmctld(job_write_lock);
+	_throttle_fini();
 
 	return NULL;
 }
@@ -996,16 +1059,30 @@ static void _spawn_cleanup_thread(
 static void _select_jobinfo_pack(select_jobinfo_t *jobinfo, Buf buffer,
 				 uint16_t protocol_version)
 {
-	if (!jobinfo) {
-		pack_bit_str(NULL, buffer);
-		pack16(0, buffer);
-		pack8(0, buffer);
-		pack_bit_str(NULL, buffer);
-	} else {
-		pack_bit_str(jobinfo->blade_map, buffer);
-		pack16(jobinfo->cleaning, buffer);
-		pack8(jobinfo->npc, buffer);
-		pack_bit_str(jobinfo->used_blades, buffer);
+	if (protocol_version >= SLURM_14_11_PROTOCOL_VERSION) {
+		if (!jobinfo) {
+			pack_bit_str_hex(NULL, buffer);
+			pack16(0, buffer);
+			pack8(0, buffer);
+			pack_bit_str_hex(NULL, buffer);
+		} else {
+			pack_bit_str_hex(jobinfo->blade_map, buffer);
+			pack16(jobinfo->cleaning, buffer);
+			pack8(jobinfo->npc, buffer);
+			pack_bit_str_hex(jobinfo->used_blades, buffer);
+		}
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		if (!jobinfo) {
+			pack_bit_str(NULL, buffer);
+			pack16(0, buffer);
+			pack8(0, buffer);
+			pack_bit_str(NULL, buffer);
+		} else {
+			pack_bit_str(jobinfo->blade_map, buffer);
+			pack16(jobinfo->cleaning, buffer);
+			pack8(jobinfo->npc, buffer);
+			pack_bit_str(jobinfo->used_blades, buffer);
+		}
 	}
 }
 
@@ -1018,10 +1095,17 @@ static int _select_jobinfo_unpack(select_jobinfo_t **jobinfo_pptr,
 
 	jobinfo->magic = JOBINFO_MAGIC;
 
-	unpack_bit_str(&jobinfo->blade_map, buffer);
-	safe_unpack16(&jobinfo->cleaning, buffer);
-	safe_unpack8(&jobinfo->npc, buffer);
-	unpack_bit_str(&jobinfo->used_blades, buffer);
+	if (protocol_version >= SLURM_14_11_PROTOCOL_VERSION) {
+		unpack_bit_str_hex(&jobinfo->blade_map, buffer);
+		safe_unpack16(&jobinfo->cleaning, buffer);
+		safe_unpack8(&jobinfo->npc, buffer);
+		unpack_bit_str_hex(&jobinfo->used_blades, buffer);
+	} else if (protocol_version >= SLURM_MIN_PROTOCOL_VERSION) {
+		unpack_bit_str(&jobinfo->blade_map, buffer);
+		safe_unpack16(&jobinfo->cleaning, buffer);
+		safe_unpack8(&jobinfo->npc, buffer);
+		unpack_bit_str(&jobinfo->used_blades, buffer);
+	}
 
 	return SLURM_SUCCESS;
 
@@ -1071,6 +1155,11 @@ extern int fini ( void )
 		_free_blade(&blade_array[i]);
 	xfree(blade_array);
 
+#if defined(HAVE_NATIVE_CRAY_GA) && !defined(HAVE_CRAY_NETWORK)
+	if (topology)
+		free(topology);
+#endif
+
 	slurm_mutex_unlock(&blade_mutex);
 
 	return other_select_fini();
@@ -1333,6 +1422,20 @@ extern int select_p_job_init(List job_list)
 			if (jobinfo->cleaning || IS_JOB_RUNNING(job_ptr))
 				_set_job_running_restore(jobinfo);
 
+			/* We need to resize bitmaps if the
+			 * blades change in count.  We must increase
+			 * the size so loops based off blade_cnt will
+			 * work correctly.
+			 */
+			if (jobinfo->blade_map
+			    && (bit_size(jobinfo->blade_map) < blade_cnt))
+				jobinfo->blade_map = bit_realloc(
+					jobinfo->blade_map, blade_cnt);
+			if (jobinfo->used_blades
+			    && (bit_size(jobinfo->used_blades) < blade_cnt))
+				jobinfo->used_blades = bit_realloc(
+					jobinfo->used_blades, blade_cnt);
+
 			if (!(slurmctld_conf.select_type_param & CR_NHC_STEP_NO)
 			    && job_ptr->step_list
 			    && list_count(job_ptr->step_list)) {
@@ -1380,26 +1483,30 @@ extern int select_p_node_init(struct node_record *node_ptr, int node_cnt)
 	int i, j;
 	uint64_t blade_id = 0;
 
-#ifdef HAVE_NATIVE_CRAY_GA
+#if defined(HAVE_NATIVE_CRAY_GA) && !defined(HAVE_CRAY_NETWORK)
 	int nn, end_nn, last_nn = 0;
 	bool found = 0;
-	alpsc_topology_t *topology = NULL;
-	size_t num_nodes;
 	char *err_msg = NULL;
 
-	if (alpsc_get_topology(&err_msg, &topology, &num_nodes)) {
-		if (err_msg) {
-			error("(%s: %d: %s) Could not get system "
-			      "topology info: %s",
-			      THIS_FILE, __LINE__, __FUNCTION__, err_msg);
-			free(err_msg);
-		} else {
-			error("(%s: %d: %s) Could not get system "
-			      "topology info: No error message present.",
-			      THIS_FILE, __LINE__, __FUNCTION__);
+	if (!topology) {
+		if (alpsc_get_topology(&err_msg, &topology,
+				       &topology_num_nodes)) {
+			if (err_msg) {
+				error("(%s: %d: %s) Could not get system "
+				      "topology info: %s",
+				      THIS_FILE, __LINE__,
+				      __FUNCTION__, err_msg);
+				free(err_msg);
+			} else {
+				error("(%s: %d: %s) Could not get system "
+				      "topology info: No error "
+				      "message present.",
+				      THIS_FILE, __LINE__, __FUNCTION__);
+			}
+			return SLURM_ERROR;
 		}
-		return SLURM_ERROR;
 	}
+
 #endif
 
 	slurm_mutex_lock(&blade_mutex);
@@ -1432,8 +1539,8 @@ extern int select_p_node_init(struct node_record *node_ptr, int node_cnt)
 			nodeinfo->nid = atoll(nid_char);
 		}
 
-#ifdef HAVE_NATIVE_CRAY_GA
-		end_nn = num_nodes;
+#if defined(HAVE_NATIVE_CRAY_GA) && !defined(HAVE_CRAY_NETWORK)
+		end_nn = topology_num_nodes;
 
 	start_again:
 
@@ -1449,7 +1556,7 @@ extern int select_p_node_init(struct node_record *node_ptr, int node_cnt)
 				break;
 			}
 		}
-		if (end_nn != num_nodes) {
+		if (end_nn != topology_num_nodes) {
 			/* already looped */
 			fatal("Node %s(%d) isn't found on the system",
 			      node_ptr->name, nodeinfo->nid);
@@ -1488,10 +1595,6 @@ extern int select_p_node_init(struct node_record *node_ptr, int node_cnt)
 	/* give back the memory */
 	xrealloc(blade_array, sizeof(blade_info_t) * blade_cnt);
 
-#ifdef HAVE_NATIVE_CRAY_GA
-	free(topology);
-#endif
-
 	slurm_mutex_unlock(&blade_mutex);
 
 	return other_node_init(node_ptr, node_cnt);
@@ -1789,27 +1892,28 @@ extern int select_p_step_finish(struct step_record *step_ptr)
 		post_job_step(step_ptr);
 		return SLURM_SUCCESS;
 	}
-	/* The NHC needs to be ran after each step even if the job is
-	   about to run the NHC for the allocation.  The NHC
-	   developers feel this is needed.  If it ever changes just
-	   remove the below commented code.
-	*/
 
-	/*  else if (IS_JOB_COMPLETING(step_ptr->job_ptr)) { */
-	/* 	debug3("step completion %u.%u was received after job " */
-	/* 	      "allocation is already completing, no extra NHC needed.", */
-	/* 	      step_ptr->job_ptr->job_id, step_ptr->step_id); */
-	/* 	other_step_finish(step_ptr); */
-	/* 	/\* free resources on the job *\/ */
-	/* 	post_job_step(step_ptr); */
-	/* 	return SLURM_SUCCESS; */
-	/* } */
+#if 0
+	/* The NHC needs to be ran after each step even if the job is about to
+	 * run the NHC for the allocation.  The NHC developers feel this is
+	 * needed.  If it ever changes just use this below code. */
+	else if (IS_JOB_COMPLETING(step_ptr->job_ptr) ||
+		 IS_JOB_FINISHED(step_ptr->job_ptr)) {
+		debug3("step completion %u.%u was received after job "
+		      "allocation is already completing, no extra NHC needed.",
+		      step_ptr->job_ptr->job_id, step_ptr->step_id);
+		other_step_finish(step_ptr);
+		/* free resources on the job */
+		post_job_step(step_ptr);
+		return SLURM_SUCCESS;
+	}
+#endif
 
-	if (jobinfo->cleaning == 1)
+	if (jobinfo->cleaning == 1) {
 		error("Cleaning flag already set for job step %u.%u, "
 		      "this should never happen.",
 		      step_ptr->step_id, step_ptr->job_ptr->job_id);
-	else {
+	} else {
 		jobinfo->cleaning = 1;
 		_spawn_cleanup_thread(step_ptr, _step_fini);
 	}
@@ -2212,11 +2316,11 @@ extern int select_p_fail_cnode(struct step_record *step_ptr)
 	return other_fail_cnode(step_ptr);
 }
 
-extern int select_p_get_info_from_plugin(enum select_jobdata_type info,
+extern int select_p_get_info_from_plugin(enum select_plugindata_info dinfo,
 					 struct job_record *job_ptr,
 					 void *data)
 {
-	return other_get_info_from_plugin(info, job_ptr, data);
+	return other_get_info_from_plugin(dinfo, job_ptr, data);
 }
 
 extern int select_p_update_node_config(int index)
@@ -2240,12 +2344,13 @@ extern int select_p_reconfigure(void)
 	return other_reconfigure();
 }
 
-extern bitstr_t * select_p_resv_test(bitstr_t *avail_bitmap, uint32_t node_cnt,
-				     uint32_t *core_cnt, bitstr_t **core_bitmap,
-				     uint32_t flags)
+extern bitstr_t * select_p_resv_test(resv_desc_msg_t *resv_desc_ptr,
+				     uint32_t node_cnt,
+				     bitstr_t *avail_bitmap,
+				     bitstr_t **core_bitmap)
 {
-	return other_resv_test(avail_bitmap, node_cnt, core_cnt, core_bitmap,
-			       flags);
+	return other_resv_test(resv_desc_ptr, node_cnt,
+			       avail_bitmap, core_bitmap);
 }
 
 extern void select_p_ba_init(node_info_msg_t *node_info_ptr, bool sanity_check)
@@ -2262,3 +2367,8 @@ extern void select_p_ba_fini(void)
 {
 	other_ba_fini();
 }
+
+extern bitstr_t *select_p_ba_cnodelist2bitmap(char *cnodelist)
+{
+	return other_ba_cnodelist2bitmap(cnodelist);
+}
