@@ -39,6 +39,7 @@
 \*****************************************************************************/
 
 #include "sacct.h"
+#include "src/common/cpu_frequency.h"
 #include "src/common/parse_time.h"
 #include "slurm.h"
 
@@ -118,8 +119,87 @@ static void _print_small_double(
 		snprintf(outbuf, buf_size, "0");
 }
 
+/* Translate bitmap representation from hex to decimal format, replacing
+ * array_task_str. */
+static void _xlate_task_str(slurmdb_job_rec_t *job_ptr)
+{
+	static int bitstr_len = -1;
+	int buf_size, len;
+	int i, i_first, i_last, i_prev, i_step = 0;
+	bitstr_t *task_bitmap;
+	char *in_buf = job_ptr->array_task_str;
+	char *out_buf = NULL;
+
+	if (!in_buf)
+		return;
+
+	i = strlen(in_buf);
+	task_bitmap = bit_alloc(i * 4);
+	bit_unfmt_hexmask(task_bitmap, in_buf);
+
+	/* Check first for a step function */
+	i_first = bit_ffs(task_bitmap);
+	i_last  = bit_fls(task_bitmap);
+	if (((i_last - i_first) > 10) &&
+	    !bit_test(task_bitmap, i_first + 1)) {
+		bool is_step = true;
+		i_prev = i_first;
+		for (i = i_first + 1; i <= i_last; i++) {
+			if (!bit_test(task_bitmap, i))
+				continue;
+			if (i_step == 0) {
+				i_step = i - i_prev;
+			} else if ((i - i_prev) != i_step) {
+				is_step = false;
+				break;
+			}
+			i_prev = i;
+		}
+		if (is_step) {
+			xstrfmtcat(out_buf, "%d-%d:%d",
+				   i_first, i_last, i_step);
+		}
+	}
+
+	if (bitstr_len > 0) {
+		/* Print the first bitstr_len bytes of the bitmap string */
+		buf_size = bitstr_len;
+		out_buf = xmalloc(buf_size);
+		bit_fmt(out_buf, buf_size, task_bitmap);
+		len = strlen(out_buf);
+		if (len > (buf_size - 3))
+			for (i = 0; i < 3; i++)
+				out_buf[buf_size - 2 - i] = '.';
+	} else {
+		/* Print the full bitmap's string representation.
+		 * For huge bitmaps this can take roughly one minute,
+		 * so let the client do the work */
+		buf_size = bit_size(task_bitmap) * 8;
+		while (1) {
+			out_buf = xmalloc(buf_size);
+			bit_fmt(out_buf, buf_size, task_bitmap);
+			len = strlen(out_buf);
+			if ((len > 0) && (len < (buf_size - 32)))
+				break;
+			xfree(out_buf);
+			buf_size *= 2;
+		}
+	}
+
+	if (job_ptr->array_max_tasks)
+		xstrfmtcat(out_buf, "%c%u", '%', job_ptr->array_max_tasks);
+
+	xfree(job_ptr->array_task_str);
+	job_ptr->array_task_str = out_buf;
+}
+
 void print_fields(type_t type, void *object)
 {
+	if (!object) {
+		fatal ("Job or step record is NULL");
+		return;
+	}
+
 	slurmdb_job_rec_t *job = (slurmdb_job_rec_t *)object;
 	slurmdb_step_rec_t *step = (slurmdb_step_rec_t *)object;
 	jobcomp_job_rec_t *job_comp = (jobcomp_job_rec_t *)object;
@@ -128,6 +208,7 @@ void print_fields(type_t type, void *object)
 	struct passwd *pw = NULL;
 	struct	group *gr = NULL;
 	char outbuf[FORMAT_STRING_SIZE];
+	bool got_stats = false;
 
 	switch(type) {
 	case JOB:
@@ -140,10 +221,14 @@ void print_fields(type_t type, void *object)
 		*/
 		if (!step)
 			job->track_steps = 1;
+		if (job->stats.cpu_min != NO_VAL)
+			got_stats = true;
 		job_comp = NULL;
 		break;
 	case JOBSTEP:
 		job = step->job_ptr;
+		if (step->stats.cpu_min != NO_VAL)
+			got_stats = true;
 		job_comp = NULL;
 		break;
 	case JOBCOMP:
@@ -156,7 +241,7 @@ void print_fields(type_t type, void *object)
 
 	list_iterator_reset(print_fields_itr);
 	while((field = list_next(print_fields_itr))) {
-		char *tmp_char = NULL;
+		char *tmp_char = NULL, id[FORMAT_STRING_SIZE];
 		int tmp_int = NO_VAL, tmp_int2 = NO_VAL;
 		double tmp_dub = (double)NO_VAL;
 		uint32_t tmp_uint32 = (uint32_t)NO_VAL;
@@ -183,6 +268,23 @@ void print_fields(type_t type, void *object)
 					     tmp_int,
 					     (curr_inx == field_count));
 			break;
+		case PRINT_ALLOC_GRES:
+			switch(type) {
+			case JOB:
+				tmp_char = job->alloc_gres;
+				break;
+			case JOBSTEP:
+				tmp_char = step->job_ptr->alloc_gres;
+				break;
+			case JOBCOMP:
+			default:
+				tmp_char = NULL;
+				break;
+			}
+			field->print_routine(field,
+					     tmp_char,
+					     (curr_inx == field_count));
+			break;
 		case PRINT_ACCOUNT:
 			switch(type) {
 			case JOB:
@@ -201,16 +303,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_ACT_CPUFREQ:
-			switch (type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch (type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub =
+							step->stats.act_cpufreq;
+					break;
+				case JOBSTEP:
 					tmp_dub = step->stats.act_cpufreq;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.act_cpufreq;
-				break;
-			default:
-				break;
+					break;
+				default:
+					break;
+				}
 			}
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				convert_num_unit2((double)tmp_dub,
@@ -239,17 +344,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVECPU:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.cpu_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.cpu_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.cpu_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.cpu_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
@@ -261,17 +368,20 @@ void print_fields(type_t type, void *object)
 			xfree(tmp_char);
 			break;
 		case PRINT_AVEDISKREAD:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.disk_read_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.disk_read_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->
+							stats.disk_read_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.disk_read_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			_print_small_double(outbuf, sizeof(outbuf),
 					    tmp_dub, UNIT_MEGA);
@@ -281,17 +391,20 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVEDISKWRITE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.disk_write_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.disk_write_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->
+							stats.disk_write_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.disk_write_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			_print_small_double(outbuf, sizeof(outbuf),
 					    tmp_dub, UNIT_MEGA);
@@ -301,17 +414,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVEPAGES:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.pages_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.pages_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.pages_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.pages_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				convert_num_unit((double)tmp_dub,
@@ -323,17 +438,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVERSS:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.rss_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.rss_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.rss_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.rss_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				convert_num_unit((double)tmp_dub,
@@ -345,18 +462,21 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_AVEVSIZE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.vsize_ave;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.vsize_ave;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.vsize_ave;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.vsize_ave;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
+
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				convert_num_unit((double)tmp_dub,
 						 outbuf, sizeof(outbuf),
@@ -415,16 +535,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_CONSUMED_ENERGY:
-			switch (type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch (type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = step->
+							stats.consumed_energy;
+					break;
+				case JOBSTEP:
 					tmp_dub = step->stats.consumed_energy;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.consumed_energy;
-				break;
-			default:
-				break;
+					break;
+				default:
+					break;
+				}
 			}
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				convert_num_unit2((double)tmp_dub,
@@ -436,16 +559,19 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_CONSUMED_ENERGY_RAW:
-			switch (type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch (type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = step->
+							stats.consumed_energy;
+					break;
+				case JOBSTEP:
 					tmp_dub = step->stats.consumed_energy;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.consumed_energy;
-				break;
-			default:
-				break;
+					break;
+				default:
+					break;
+				}
 			}
 
 			field->print_routine(field,
@@ -491,7 +617,6 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_DERIVED_EC:
-			tmp_int = 0;
 			tmp_int2 = 0;
 			switch(type) {
 			case JOB:
@@ -641,6 +766,53 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_JOBID:
+			if (type == JOBSTEP)
+				job = step->job_ptr;
+
+			if (job) {
+				if (job->array_task_str) {
+					_xlate_task_str(job);
+					snprintf(id, FORMAT_STRING_SIZE,
+						 "%u_[%s]",
+						 job->array_job_id,
+						 job->array_task_str);
+				} else if (job->array_task_id != NO_VAL)
+					snprintf(id, FORMAT_STRING_SIZE,
+						 "%u_%u",
+						 job->array_job_id,
+						 job->array_task_id);
+				else
+					snprintf(id, FORMAT_STRING_SIZE,
+						 "%u",
+						 job->jobid);
+			}
+
+			switch(type) {
+			case JOB:
+				tmp_char = xstrdup(id);
+				break;
+			case JOBSTEP:
+				if (step->stepid == NO_VAL)
+					tmp_char = xstrdup_printf(
+						"%s.batch", id);
+				else
+					tmp_char = xstrdup_printf(
+						"%s.%u",
+						id, step->stepid);
+				break;
+			case JOBCOMP:
+				tmp_char = xstrdup_printf("%u",
+							  job_comp->jobid);
+				break;
+			default:
+				break;
+			}
+			field->print_routine(field,
+					     tmp_char,
+					     (curr_inx == field_count));
+			xfree(tmp_char);
+			break;
+		case PRINT_JOBIDRAW:
 			switch(type) {
 			case JOB:
 				tmp_char = xstrdup_printf("%u", job->jobid);
@@ -711,17 +883,20 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXDISKREAD:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.disk_read_max;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.disk_read_max;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->
+							stats.disk_read_max;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.disk_read_max;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			_print_small_double(outbuf, sizeof(outbuf),
 					    tmp_dub, UNIT_MEGA);
@@ -731,22 +906,26 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXDISKREADNODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							disk_read_max_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-						job->stats.disk_read_max_nodeid,
-						job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.disk_read_max_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.
+						disk_read_max_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
 			field->print_routine(field,
 					     tmp_char,
@@ -754,19 +933,23 @@ void print_fields(type_t type, void *object)
 			xfree(tmp_char);
 			break;
 		case PRINT_MAXDISKREADTASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 =
-						job->stats.disk_read_max_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.disk_read_max_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 =
+							job->stats.
+							disk_read_max_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.
+						disk_read_max_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
 			}
 			if (tmp_uint32 == (uint32_t)NO_VAL)
 				tmp_uint32 = NO_VAL;
@@ -775,17 +958,20 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXDISKWRITE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.disk_write_max;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.disk_write_max;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.
+							disk_write_max;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.disk_write_max;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
 			}
 			_print_small_double(outbuf, sizeof(outbuf),
 					    tmp_dub, UNIT_MEGA);
@@ -795,22 +981,26 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXDISKWRITENODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							disk_write_max_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-					   job->stats.disk_write_max_nodeid,
-					   job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.disk_write_max_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.
+						disk_write_max_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
 			field->print_routine(field,
 					     tmp_char,
@@ -818,65 +1008,75 @@ void print_fields(type_t type, void *object)
 			xfree(tmp_char);
 			break;
 		case PRINT_MAXDISKWRITETASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 =
-					   job->stats.disk_write_max_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.disk_write_max_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 =
+							job->stats.
+							disk_write_max_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.
+						disk_write_max_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
+				if (tmp_uint32 == (uint32_t)NO_VAL)
+					tmp_uint32 = NO_VAL;
 			}
-			if (tmp_uint32 == (uint32_t)NO_VAL)
-				tmp_uint32 = NO_VAL;
 			field->print_routine(field,
 					     tmp_uint32,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXPAGES:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint64 = job->stats.pages_max;
-				break;
-			case JOBSTEP:
-				tmp_uint64 = step->stats.pages_max;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint64 =
+							job->stats.pages_max;
+					break;
+				case JOBSTEP:
+					tmp_uint64 = step->stats.pages_max;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
+				if (tmp_uint64 != (uint64_t)NO_VAL)
+					convert_num_unit((double)tmp_uint64,
+							 outbuf, sizeof(outbuf),
+							 UNIT_KILO);
 			}
-			if (tmp_uint64 != (uint64_t)NO_VAL)
-				convert_num_unit((double)tmp_uint64,
-						 outbuf, sizeof(outbuf),
-						 UNIT_KILO);
 
 			field->print_routine(field,
 					     outbuf,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXPAGESNODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							pages_max_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-						job->stats.pages_max_nodeid,
-						job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.pages_max_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.pages_max_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
 			field->print_routine(field,
 					     tmp_char,
@@ -884,217 +1084,257 @@ void print_fields(type_t type, void *object)
 			xfree(tmp_char);
 			break;
 		case PRINT_MAXPAGESTASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 =
-						job->stats.pages_max_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.pages_max_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 =
+							job->stats.
+							pages_max_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.
+						pages_max_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
+				if (tmp_uint32 == (uint32_t)NO_VAL)
+					tmp_uint32 = NO_VAL;
 			}
-			if (tmp_uint32 == (uint32_t)NO_VAL)
-				tmp_uint32 = NO_VAL;
+
 			field->print_routine(field,
 					     tmp_uint32,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXRSS:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint64 = job->stats.rss_max;
-				break;
-			case JOBSTEP:
-				tmp_uint64 = step->stats.rss_max;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint64 = job->stats.rss_max;
+					break;
+				case JOBSTEP:
+					tmp_uint64 = step->stats.rss_max;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
+				if (tmp_uint64 != (uint64_t)NO_VAL)
+					convert_num_unit((double)tmp_uint64,
+							 outbuf, sizeof(outbuf),
+							 UNIT_KILO);
 			}
-			if (tmp_uint64 != (uint64_t)NO_VAL)
-				convert_num_unit((double)tmp_uint64,
-						 outbuf, sizeof(outbuf),
-						 UNIT_KILO);
 
 			field->print_routine(field,
 					     outbuf,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXRSSNODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							rss_max_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-						job->stats.rss_max_nodeid,
-						job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.rss_max_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.rss_max_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
+
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
 			xfree(tmp_char);
 			break;
 		case PRINT_MAXRSSTASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 = job->stats.rss_max_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.rss_max_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 = job->stats.
+							rss_max_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.rss_max_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
+				if (tmp_uint32 == (uint32_t)NO_VAL)
+					tmp_uint32 = NO_VAL;
 			}
-			if (tmp_uint32 == (uint32_t)NO_VAL)
-				tmp_uint32 = NO_VAL;
+
 			field->print_routine(field,
 					     tmp_uint32,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXVSIZE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint64 = job->stats.vsize_max;
-				break;
-			case JOBSTEP:
-				tmp_uint64 = step->stats.vsize_max;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint64 = (uint64_t)NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint64 = job->stats.
+							vsize_max;
+					break;
+				case JOBSTEP:
+					tmp_uint64 = step->stats.vsize_max;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint64 = (uint64_t)NO_VAL;
+					break;
+				}
+
+				if (tmp_uint64 != (uint64_t)NO_VAL)
+					convert_num_unit((double)tmp_uint64,
+							 outbuf, sizeof(outbuf),
+							 UNIT_KILO);
 			}
-			if (tmp_uint64 != (uint64_t)NO_VAL)
-				convert_num_unit((double)tmp_uint64,
-						 outbuf, sizeof(outbuf),
-						 UNIT_KILO);
 
 			field->print_routine(field,
 					     outbuf,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MAXVSIZENODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							vsize_max_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-						job->stats.vsize_max_nodeid,
-						job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.vsize_max_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.vsize_max_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
+
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
 			xfree(tmp_char);
 			break;
 		case PRINT_MAXVSIZETASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 =
-						job->stats.vsize_max_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.vsize_max_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 =
+							job->stats.
+							vsize_max_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.
+						vsize_max_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
+				if (tmp_uint32 == (uint32_t)NO_VAL)
+					tmp_uint32 = NO_VAL;
 			}
-			if (tmp_uint32 == (uint32_t)NO_VAL)
-				tmp_uint32 = NO_VAL;
+
 			field->print_routine(field,
 					     tmp_uint32,
 					     (curr_inx == field_count));
 			break;
 		case PRINT_MINCPU:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_dub = job->stats.cpu_min;
-				break;
-			case JOBSTEP:
-				tmp_dub = step->stats.cpu_min;
-				break;
-			case JOBCOMP:
-			default:
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_dub = job->stats.cpu_min;
+					break;
+				case JOBSTEP:
+					tmp_dub = step->stats.cpu_min;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
+				if (!fuzzy_equal(tmp_dub, NO_VAL))
+					tmp_char = _elapsed_time(
+						(long)tmp_dub, 0);
 			}
-			if (!fuzzy_equal(tmp_dub, NO_VAL))
-				tmp_char = _elapsed_time((long)tmp_dub, 0);
+
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
 			xfree(tmp_char);
 			break;
 		case PRINT_MINCPUNODE:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_char = find_hostname(
+							job->stats.
+							cpu_min_nodeid,
+							job->nodes);
+					break;
+				case JOBSTEP:
 					tmp_char = find_hostname(
-						job->stats.cpu_min_nodeid,
-						job->nodes);
-				break;
-			case JOBSTEP:
-				tmp_char = find_hostname(
-					step->stats.cpu_min_nodeid,
-					step->nodes);
-				break;
-			case JOBCOMP:
-			default:
-				tmp_char = NULL;
-				break;
+						step->stats.cpu_min_nodeid,
+						step->nodes);
+					break;
+				case JOBCOMP:
+				default:
+					tmp_char = NULL;
+					break;
+				}
 			}
+
 			field->print_routine(field,
 					     tmp_char,
 					     (curr_inx == field_count));
 			xfree(tmp_char);
 			break;
 		case PRINT_MINCPUTASK:
-			switch(type) {
-			case JOB:
-				if (!job->track_steps)
-					tmp_uint32 = job->stats.cpu_min_taskid;
-				break;
-			case JOBSTEP:
-				tmp_uint32 = step->stats.cpu_min_taskid;
-				break;
-			case JOBCOMP:
-			default:
-				tmp_uint32 = NO_VAL;
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					if (!job->track_steps)
+						tmp_uint32 = job->stats.
+							cpu_min_taskid;
+					break;
+				case JOBSTEP:
+					tmp_uint32 = step->stats.cpu_min_taskid;
+					break;
+				case JOBCOMP:
+				default:
+					tmp_uint32 = NO_VAL;
+					break;
+				}
+				if (tmp_uint32 == (uint32_t)NO_VAL)
+					tmp_uint32 = NO_VAL;
 			}
-			if (tmp_uint32 == (uint32_t)NO_VAL)
-				tmp_uint32 = NO_VAL;
+
 			field->print_routine(field,
 					     tmp_uint32,
 					     (curr_inx == field_count));
@@ -1267,18 +1507,7 @@ void print_fields(type_t type, void *object)
 			default:
 				break;
 			}
-			if (tmp_dub == CPU_FREQ_LOW)
-				snprintf(outbuf, sizeof(outbuf), "Low");
-			else if (tmp_dub == CPU_FREQ_MEDIUM)
-				snprintf(outbuf, sizeof(outbuf), "Medium");
-			else if (tmp_dub == CPU_FREQ_HIGH)
-				snprintf(outbuf, sizeof(outbuf), "High");
-			else if (tmp_dub == CPU_FREQ_HIGHM1)
-				snprintf(outbuf, sizeof(outbuf), "Highm1");
-			else if (!fuzzy_equal(tmp_dub, NO_VAL))
-				convert_num_unit2((double)tmp_dub,
-						  outbuf, sizeof(outbuf),
-						  UNIT_KILO, 1000, false);
+			cpu_freq_to_string(outbuf, sizeof(outbuf), tmp_dub);
 			field->print_routine(field,
 					     outbuf,
 					     (curr_inx == field_count));
@@ -1302,6 +1531,23 @@ void print_fields(type_t type, void *object)
 					     tmp_int,
 					     (curr_inx == field_count));
 			break;
+		case PRINT_REQ_GRES:
+			switch(type) {
+			case JOB:
+				tmp_char = job->req_gres;
+				break;
+			case JOBSTEP:
+				tmp_char = step->job_ptr->req_gres;
+				break;
+			case JOBCOMP:
+			default:
+				tmp_char = NULL;
+				break;
+			}
+			field->print_routine(field,
+					     tmp_char,
+					     (curr_inx == field_count));
+			break;
 		case PRINT_REQ_MEM:
 			switch(type) {
 			case JOB:
@@ -1334,6 +1580,53 @@ void print_fields(type_t type, void *object)
 					     outbuf,
 					     (curr_inx == field_count));
 			break;
+		case PRINT_RESERVATION:
+			switch(type) {
+			case JOB:
+				if (job->resv_name) {
+					tmp_char = job->resv_name;
+				} else {
+					tmp_char = NULL;
+				}
+				break;
+			case JOBSTEP:
+				tmp_char = NULL;
+				break;
+			case JOBCOMP:
+				tmp_char = NULL;
+				break;
+			default:
+				tmp_char = NULL;
+				break;
+			}
+			field->print_routine(field,
+						tmp_char,
+						(curr_inx == field_count));
+			break;
+		case PRINT_RESERVATION_ID:
+			switch(type) {
+			case JOB:
+				if (job->resvid)
+					tmp_uint32 = job->resvid;
+				else
+					tmp_uint32 = NO_VAL;
+				break;
+			case JOBSTEP:
+				tmp_uint32 = NO_VAL;
+				break;
+			case JOBCOMP:
+				tmp_uint32 = NO_VAL;
+				break;
+			default:
+				tmp_uint32 = NO_VAL;
+				break;
+			}
+			if (tmp_uint32 == (uint32_t)NO_VAL)
+				tmp_uint32 = NO_VAL;
+			field->print_routine(field,
+						tmp_uint32,
+						(curr_inx == field_count));
+			break;
 		case PRINT_RESV:
 			switch(type) {
 			case JOB:
@@ -1496,23 +1789,22 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_SYSTEMCPU:
-			switch(type) {
-			case JOB:
-				tmp_int = job->sys_cpu_sec;
-				tmp_int2 = job->sys_cpu_usec;
-				break;
-			case JOBSTEP:
-				tmp_int = step->sys_cpu_sec;
-				tmp_int2 = step->sys_cpu_usec;
-				break;
-			case JOBCOMP:
-
-				break;
-			default:
-
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					tmp_int = job->sys_cpu_sec;
+					tmp_int2 = job->sys_cpu_usec;
+					break;
+				case JOBSTEP:
+					tmp_int = step->sys_cpu_sec;
+					tmp_int2 = step->sys_cpu_usec;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
+				tmp_char = _elapsed_time(tmp_int, tmp_int2);
 			}
-			tmp_char = _elapsed_time(tmp_int, tmp_int2);
 
 			field->print_routine(field,
 					     tmp_char,
@@ -1619,23 +1911,22 @@ void print_fields(type_t type, void *object)
 					     (curr_inx == field_count));
 			break;
 		case PRINT_USERCPU:
-			switch(type) {
-			case JOB:
-				tmp_int = job->user_cpu_sec;
-				tmp_int2 = job->user_cpu_usec;
-				break;
-			case JOBSTEP:
-				tmp_int = step->user_cpu_sec;
-				tmp_int2 = step->user_cpu_usec;
-				break;
-			case JOBCOMP:
-
-				break;
-			default:
-
-				break;
+			if (got_stats) {
+				switch(type) {
+				case JOB:
+					tmp_int = job->user_cpu_sec;
+					tmp_int2 = job->user_cpu_usec;
+					break;
+				case JOBSTEP:
+					tmp_int = step->user_cpu_sec;
+					tmp_int2 = step->user_cpu_usec;
+					break;
+				case JOBCOMP:
+				default:
+					break;
+				}
+				tmp_char = _elapsed_time(tmp_int, tmp_int2);
 			}
-			tmp_char = _elapsed_time(tmp_int, tmp_int2);
 
 			field->print_routine(field,
 					     tmp_char,
