@@ -83,6 +83,7 @@ static void _load_job_records (void);
 static int  _multi_cluster(List clusters);
 static int  _proc_cluster(void);
 static int  _verify_job_ids (void);
+static int  _signal_job_by_str(void);
 
 static job_info_msg_t * job_buffer_ptr = NULL;
 
@@ -122,7 +123,7 @@ main (int argc, char *argv[])
 	else
 		rc = _proc_cluster();
 
-	exit (rc);
+	exit(rc);
 }
 
 /* _multi_cluster - process job cancellation across a list of clusters */
@@ -149,6 +150,11 @@ _proc_cluster(void)
 	int filter_cnt = 0;
 	int rc;
 
+	if (has_default_opt()) {
+		rc = _signal_job_by_str();
+		return rc;
+	}
+
 	_load_job_records();
 	rc = _verify_job_ids();
 	if ((opt.account) ||
@@ -169,7 +175,9 @@ _proc_cluster(void)
 	return rc;
 }
 
-/* _load_job_records - load all job information for filtering and verification */
+/* _load_job_records - load all job information for filtering
+ * and verification
+ */
 static void
 _load_job_records (void)
 {
@@ -222,18 +230,29 @@ _verify_job_ids (void)
 				break;
 		}
 		jp = &job_ptr[i];
-		if (((IS_JOB_FINISHED(jp)) ||
-		     (i >= job_buffer_ptr->record_count)) &&
-		     (job_ptr[i].array_task_id == NO_VAL) &&
-		     (opt.verbose >= 0)) {
-			if (opt.step_id[j] == SLURM_BATCH_SCRIPT)
+		if ((i >= job_buffer_ptr->record_count) ||
+		    IS_JOB_FINISHED(jp)) {
+			if (opt.verbose < 0) {
+				;
+			} else if ((opt.array_id[j] == NO_VAL) &&
+				   (opt.step_id[j] == SLURM_BATCH_SCRIPT)) {
 				error("Kill job error on job id %u: %s",
 				      opt.job_id[j],
 				      slurm_strerror(ESLURM_INVALID_JOB_ID));
-			else
+			} else if (opt.array_id[j] == NO_VAL) {
 				error("Kill job error on job step id %u.%u: %s",
 				      opt.job_id[j], opt.step_id[j],
 				      slurm_strerror(ESLURM_INVALID_JOB_ID));
+			} else if (opt.step_id[j] == SLURM_BATCH_SCRIPT) {
+				error("Kill job error on job id %u_%u: %s",
+				      opt.job_id[j], opt.array_id[j],
+				      slurm_strerror(ESLURM_INVALID_JOB_ID));
+			} else {
+				error("Kill job error on job step id %u_%u.%u: %s",
+				      opt.job_id[j], opt.array_id[j],
+				      opt.step_id[j],
+				      slurm_strerror(ESLURM_INVALID_JOB_ID));
+			}
 			rc = 1;
 		}
 	}
@@ -546,7 +565,7 @@ _cancel_job_id (void *ci)
 	uint32_t job_id = cancel_info->job_id;
 	uint32_t array_job_id  = cancel_info->array_job_id;
 	uint32_t array_task_id = cancel_info->array_task_id;
-	uint16_t sig    = cancel_info->sig;
+	uint32_t sig    = cancel_info->sig;
 
 	if (sig == (uint16_t)-1) {
 		sig = SIGKILL;
@@ -578,6 +597,7 @@ _cancel_job_id (void *ci)
 			error_code = slurm_kill_job (job_id, sig, flags);
 		} else {
 			if (opt.batch) {
+				sig = sig | (KILL_JOB_BATCH << 24);
 				error_code = slurm_signal_job_step(job_id,
 						SLURM_BATCH_SCRIPT, sig);
 			} else {
@@ -723,3 +743,26 @@ _confirmation (int i, uint32_t step_id)
 	}
 
 }
+
+/* _signal_job_by_str()
+ */
+static int
+_signal_job_by_str(void)
+{
+	int cc, i;
+	int rc = 0;
+
+	if (opt.signal == (uint16_t) - 1)
+		opt.signal = SIGKILL;
+
+	for (i = 0; opt.job_list[i]; i++) {
+		verbose("Terminating job %s", opt.job_list[i]);
+
+		cc = slurm_kill_job2(opt.job_list[i], opt.signal, 0);
+		if ((cc != SLURM_SUCCESS) && (opt.verbose != -1)) {
+			error("slurm_kill_job2() failed %s", slurm_strerror(errno));
+			rc = -1;
+		}
+	}
+	return rc;
+}
