@@ -112,7 +112,7 @@ static void _adjust_limit_usage(int type, struct job_record *job_ptr)
 {
 	slurmdb_association_rec_t *assoc_ptr = NULL;
 	assoc_mgr_lock_t locks = { WRITE_LOCK, NO_LOCK,
-				   WRITE_LOCK, NO_LOCK, NO_LOCK };
+				   WRITE_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 	uint64_t used_cpu_run_secs = 0;
 	uint32_t job_memory = 0;
 	uint32_t node_cnt;
@@ -381,7 +381,7 @@ extern void acct_policy_alter_job(struct job_record *job_ptr,
 {
 	slurmdb_association_rec_t *assoc_ptr = NULL;
 	assoc_mgr_lock_t locks = { WRITE_LOCK, NO_LOCK,
-				   WRITE_LOCK, NO_LOCK, NO_LOCK };
+				   WRITE_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 	uint64_t used_cpu_run_secs, new_used_cpu_run_secs;
 
 	if (!IS_JOB_RUNNING(job_ptr) || (job_ptr->time_limit == new_time_limit))
@@ -437,7 +437,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 				 struct part_record *part_ptr,
 				 slurmdb_association_rec_t *assoc_in,
 				 slurmdb_qos_rec_t *qos_ptr,
-				 uint16_t *reason,
+				 uint32_t *reason,
 				 acct_policy_limit_set_t *acct_policy_limit_set,
 				 bool update_call)
 {
@@ -452,7 +452,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 	uint32_t job_memory = 0;
 	bool admin_set_memory_limit = false;
 	assoc_mgr_lock_t locks = { READ_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+				   READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
+	bool strict_checking;
 
 	xassert(acct_policy_limit_set);
 
@@ -494,7 +495,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 
 	if (qos_ptr) {
 		slurmdb_used_limits_t *used_limits = NULL;
-		bool strict_checking =
+		strict_checking =
 			(reason || (qos_ptr->flags & QOS_FLAG_DENY_LIMIT));
 		/* for validation we don't need to look at
 		 * qos_ptr->grp_cpu_mins.
@@ -509,7 +510,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		} else if (strict_checking && (job_desc->min_cpus != NO_VAL)
 			   && (job_desc->min_cpus > qos_ptr->max_cpus_pu)) {
 			if (reason)
-				*reason = WAIT_QOS_RESOURCE_LIMIT;
+				*reason = WAIT_QOS_MAX_CPU_PER_USER;
 
 			debug2("job submit for user %s(%u): "
 			       "min cpu request %u exceeds "
@@ -524,7 +525,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		} else if (strict_checking && (job_desc->min_cpus != NO_VAL)
 			   && (job_desc->min_cpus > qos_ptr->grp_cpus)) {
 			if (reason)
-				*reason = WAIT_QOS_RESOURCE_LIMIT;
+				*reason = WAIT_QOS_GRP_CPU;
+
 			debug2("job submit for user %s(%u): "
 			       "min cpu request %u exceeds "
 			       "group max cpu limit %u for qos '%s'",
@@ -544,7 +546,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    && (qos_ptr->grp_mem != INFINITE)
 		    && (job_memory > qos_ptr->grp_mem)) {
 			if (reason)
-				*reason = WAIT_QOS_JOB_LIMIT;
+				*reason = WAIT_QOS_GRP_MEMORY;
 			debug2("job submit for user %s(%u): "
 			       "min memory request %u exceeds "
 			       "group max memory limit %u for qos '%s'",
@@ -566,8 +568,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 			/* no need to check/set */
 		} else if (strict_checking && (job_desc->min_nodes != NO_VAL)
 			   && (job_desc->min_nodes > qos_ptr->max_nodes_pu)) {
+			/* MaxNodesPerUser
+			 */
 			if (reason)
-				*reason = WAIT_QOS_RESOURCE_LIMIT;
+				*reason = WAIT_QOS_MAX_NODE_PER_USER;
 			debug2("job submit for user %s(%u): "
 			       "min node request %u exceeds "
 			       "per-user max node limit %u for qos '%s'",
@@ -581,7 +585,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		} else if (strict_checking && (job_desc->min_nodes != NO_VAL)
 			   && (job_desc->min_nodes > qos_ptr->grp_nodes)) {
 			if (reason)
-				*reason = WAIT_QOS_JOB_LIMIT;
+				*reason = WAIT_QOS_GRP_NODES;
 			debug2("job submit for user %s(%u): "
 			       "min node request %u exceeds "
 			       "group max node limit %u for qos '%s'",
@@ -597,6 +601,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		if ((qos_ptr->grp_submit_jobs != INFINITE) &&
 		    ((qos_ptr->usage->grp_used_submit_jobs + job_cnt)
 		     > qos_ptr->grp_submit_jobs)) {
+			if (reason)
+				*reason = WAIT_QOS_GRP_SUB_JOB;
 			debug2("job submit for user %s(%u): "
 			       "group max submit job limit exceeded %u "
 			       "for qos '%s'",
@@ -635,7 +641,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		} else if (strict_checking && (job_desc->min_cpus != NO_VAL)
 			   && (job_desc->min_cpus > qos_ptr->max_cpus_pj)) {
 			if (reason)
-				*reason = WAIT_QOS_JOB_LIMIT;
+				*reason = WAIT_QOS_MAX_CPUS_PER_JOB;
 			debug2("job submit for user %s(%u): "
 			       "min cpu limit %u exceeds "
 			       "qos max %u",
@@ -658,7 +664,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		} else if (strict_checking && (job_desc->min_nodes != NO_VAL)
 			   && (job_desc->min_nodes > qos_ptr->max_nodes_pj)) {
 			if (reason)
-				*reason = WAIT_QOS_JOB_LIMIT;
+				*reason = WAIT_QOS_MAX_NODE_PER_JOB;
 			debug2("job submit for user %s(%u): "
 			       "min node limit %u exceeds "
 			       "qos max %u",
@@ -680,6 +686,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 			    (used_limits &&
 			     ((used_limits->submit_jobs + job_cnt) >
 			      qos_ptr->max_submit_jobs_pu))) {
+				if (reason)
+					*reason = WAIT_QOS_MAX_SUB_JOB;
 				debug2("job submit for user %s(%u): "
 				       "qos max submit job limit exceeded %u",
 				       user_name,
@@ -714,7 +722,7 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 			} else if (strict_checking
 				   && job_desc->time_limit > qos_time_limit) {
 				if (reason)
-					*reason = WAIT_QOS_JOB_LIMIT;
+					*reason = WAIT_QOS_MAX_WALL_PER_JOB;
 				debug2("job submit for user %s(%u): "
 				       "time limit %u exceeds qos max %u",
 				       user_name,
@@ -724,6 +732,23 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 				goto end_it;
 			}
 		}
+
+		if (strict_checking && (qos_ptr->min_cpus_pj != INFINITE)
+		    && (job_desc->min_cpus < qos_ptr->min_cpus_pj)) {
+			if (reason)
+				*reason = WAIT_QOS_MIN_CPUS;
+			debug2("job submit for user %s(%u): "
+			       "min cpus %u below "
+			       "qos min %u",
+			       user_name,
+			       job_desc->user_id,
+			       job_desc->min_cpus,
+			      qos_ptr->min_cpus_pj);
+			rc = false;
+			goto end_it;
+		}
+	} else {
+		strict_checking = reason ? true : false;
 	}
 
 	while (assoc_ptr) {
@@ -736,8 +761,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    || (assoc_ptr->grp_cpus == INFINITE)
 		    || (update_call && (job_desc->max_cpus == NO_VAL))) {
 			/* no need to check/set */
-		} else if ((job_desc->min_cpus != NO_VAL)
+		} else if (strict_checking && (job_desc->min_cpus != NO_VAL)
 			   && (job_desc->min_cpus > assoc_ptr->grp_cpus)) {
+			if (reason)
+				*reason = WAIT_ASSOC_GRP_CPU;
 			debug2("job submit for user %s(%u): "
 			       "min cpu request %u exceeds "
 			       "group max cpu limit %u for account %s",
@@ -753,10 +780,12 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		/* for validation we don't need to look at
 		 * assoc_ptr->grp_jobs.
 		 */
-		if (!admin_set_memory_limit
+		if (strict_checking && !admin_set_memory_limit
 		    && (!qos_ptr || (qos_ptr->grp_mem == INFINITE))
 		    && (assoc_ptr->grp_mem != INFINITE)
 		    && (job_memory > assoc_ptr->grp_mem)) {
+			if (reason)
+				*reason = WAIT_ASSOC_GRP_MEMORY;
 			debug2("job submit for user %s(%u): "
 			       "min memory request %u exceeds "
 			       "group max memory limit %u for account %s",
@@ -774,8 +803,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    || (assoc_ptr->grp_nodes == INFINITE)
 		    || (update_call && (job_desc->max_nodes == NO_VAL))) {
 			/* no need to check/set */
-		} else if ((job_desc->min_nodes != NO_VAL)
+		} else if (strict_checking && (job_desc->min_nodes != NO_VAL)
 			   && (job_desc->min_nodes > assoc_ptr->grp_nodes)) {
+			if (reason)
+				*reason = WAIT_ASSOC_GRP_NODES;
 			debug2("job submit for user %s(%u): "
 			       "min node request %u exceeds "
 			       "group max node limit %u for account %s",
@@ -793,6 +824,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    (assoc_ptr->grp_submit_jobs != INFINITE) &&
 		    ((assoc_ptr->usage->used_submit_jobs + job_cnt)
 		     > assoc_ptr->grp_submit_jobs)) {
+			if (reason)
+				*reason = WAIT_ASSOC_GRP_SUB_JOB;
 			debug2("job submit for user %s(%u): "
 			       "group max submit job limit exceeded %u "
 			       "for account '%s'",
@@ -827,8 +860,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    || (assoc_ptr->max_cpus_pj == INFINITE)
 		    || (update_call && (job_desc->max_cpus == NO_VAL))) {
 			/* no need to check/set */
-		} else if ((job_desc->min_cpus != NO_VAL)
+		} else if (strict_checking && (job_desc->min_cpus != NO_VAL)
 			   && (job_desc->min_cpus > assoc_ptr->max_cpus_pj)) {
+			if (reason)
+				*reason = WAIT_ASSOC_MAX_CPUS_PER_JOB;
 			debug2("job submit for user %s(%u): "
 			       "min cpu limit %u exceeds "
 			       "account max %u",
@@ -849,8 +884,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    || (assoc_ptr->max_nodes_pj == INFINITE)
 		    || (update_call && (job_desc->max_nodes == NO_VAL))) {
 			/* no need to check/set */
-		} else if ((job_desc->min_nodes != NO_VAL)
+		} else if (strict_checking && (job_desc->min_nodes != NO_VAL)
 			   && (job_desc->min_nodes > assoc_ptr->max_nodes_pj)) {
+			if (reason)
+				*reason = WAIT_ASSOC_MAX_NODE_PER_JOB;
 			debug2("job submit for user %s(%u): "
 			       "min node limit %u exceeds "
 			       "account max %u",
@@ -867,6 +904,8 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 		    (assoc_ptr->max_submit_jobs != INFINITE) &&
 		    ((assoc_ptr->usage->used_submit_jobs + job_cnt)
 		     > assoc_ptr->max_submit_jobs)) {
+			if (reason)
+				*reason = WAIT_ASSOC_MAX_SUB_JOB;
 			debug2("job submit for user %s(%u): "
 			       "account max submit job limit exceeded %u",
 			       user_name,
@@ -894,7 +933,10 @@ extern bool acct_policy_validate(job_desc_msg_t *job_desc,
 			} else if (acct_policy_limit_set->time &&
 				   job_desc->time_limit > time_limit) {
 				job_desc->time_limit = time_limit;
-			} else if (job_desc->time_limit > time_limit) {
+			} else if (strict_checking &&
+				   (job_desc->time_limit > time_limit)) {
+				if (reason)
+					*reason = WAIT_ASSOC_MAX_WALL_PER_JOB;
 				debug2("job submit for user %s(%u): "
 				       "time limit %u exceeds account max %u",
 				       user_name,
@@ -920,11 +962,13 @@ end_it:
  */
 extern bool acct_policy_job_runnable_state(struct job_record *job_ptr)
 {
-	if ((job_ptr->state_reason == WAIT_ASSOC_JOB_LIMIT) ||
+	/* If any more limits are added this will need to be added to */
+	if ((job_ptr->state_reason >= WAIT_QOS_GRP_CPU
+	     && job_ptr->state_reason <= WAIT_ASSOC_MAX_SUB_JOB) ||
+	    (job_ptr->state_reason == WAIT_ASSOC_JOB_LIMIT) ||
 	    (job_ptr->state_reason == WAIT_ASSOC_RESOURCE_LIMIT) ||
 	    (job_ptr->state_reason == WAIT_ASSOC_TIME_LIMIT) ||
 	    (job_ptr->state_reason == WAIT_QOS_JOB_LIMIT) ||
-	    (job_ptr->state_reason == WAIT_QOS_RESOURCE_LIMIT) ||
 	    (job_ptr->state_reason == WAIT_QOS_TIME_LIMIT)) {
 		return false;
 	}
@@ -952,7 +996,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 			 * parent or not
 			 */
 	assoc_mgr_lock_t locks = { READ_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+				   READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 
 	/* check to see if we are enforcing associations */
 	if (!accounting_enforce)
@@ -1001,7 +1045,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 		if ((qos_ptr->grp_jobs != INFINITE) &&
 		    (qos_ptr->usage->grp_used_jobs >= qos_ptr->grp_jobs)) {
 			xfree(job_ptr->state_desc);
-			job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+			job_ptr->state_reason = WAIT_QOS_GRP_JOB;
 			debug2("job %u being held, "
 			       "the job is at or exceeds "
 			       "group max jobs limit %u with %u for qos %s",
@@ -1022,7 +1066,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 		if ((qos_ptr->grp_wall != INFINITE)
 		    && (wall_mins >= qos_ptr->grp_wall)) {
 			xfree(job_ptr->state_desc);
-			job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+			job_ptr->state_reason = WAIT_QOS_GRP_WALL;
 			debug2("job %u being held, "
 			       "the job is at or exceeds "
 			       "group wall limit %u "
@@ -1038,13 +1082,15 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 
 		/* we don't need to check max_cpus_pj here */
 
+		/* we don't need to check min_cpus_pj here */
+
 		/* we don't need to check max_cpus_pu here */
 
 		if (qos_ptr->max_jobs_pu != INFINITE) {
 			if (used_limits->jobs >= qos_ptr->max_jobs_pu) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_MAX_JOB_PER_USER;
 				debug2("job %u being held, "
 				       "the job is at or exceeds "
 				       "max jobs per-user limit "
@@ -1072,7 +1118,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 			    (job_ptr->time_limit > time_limit)) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_JOB_LIMIT;
+					WAIT_QOS_MAX_WALL_PER_JOB;
 				debug2("job %u being held, "
 				       "time limit %u exceeds qos "
 				       "max wall pj %u",
@@ -1104,7 +1150,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 		    (assoc_ptr->grp_jobs != INFINITE) &&
 		    (assoc_ptr->usage->used_jobs >= assoc_ptr->grp_jobs)) {
 			xfree(job_ptr->state_desc);
-			job_ptr->state_reason = WAIT_ASSOC_RESOURCE_LIMIT;
+			job_ptr->state_reason = WAIT_ASSOC_GRP_JOB;
 			debug2("job %u being held, "
 			       "assoc %u is at or exceeds "
 			       "group max jobs limit %u with %u for account %s",
@@ -1127,7 +1173,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 		    && (assoc_ptr->grp_wall != INFINITE)
 		    && (wall_mins >= assoc_ptr->grp_wall)) {
 			xfree(job_ptr->state_desc);
-			job_ptr->state_reason = WAIT_ASSOC_RESOURCE_LIMIT;
+			job_ptr->state_reason = WAIT_ASSOC_GRP_WALL;
 			debug2("job %u being held, "
 			       "assoc %u is at or exceeds "
 			       "group wall limit %u "
@@ -1158,7 +1204,7 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 		    (assoc_ptr->max_jobs != INFINITE) &&
 		    (assoc_ptr->usage->used_jobs >= assoc_ptr->max_jobs)) {
 			xfree(job_ptr->state_desc);
-			job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+			job_ptr->state_reason = WAIT_ASSOC_MAX_JOBS;
 			debug2("job %u being held, "
 			       "assoc %u is at or exceeds "
 			       "max jobs limit %u with %u for account %s",
@@ -1183,7 +1229,8 @@ extern bool acct_policy_job_runnable_pre_select(struct job_record *job_ptr)
 			if ((job_ptr->time_limit != NO_VAL) &&
 			    (job_ptr->time_limit > time_limit)) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason =
+					WAIT_ASSOC_MAX_WALL_PER_JOB;
 				debug2("job %u being held, "
 				       "time limit %u exceeds account max %u",
 				       job_ptr->job_id,
@@ -1230,7 +1277,7 @@ extern bool acct_policy_job_runnable_post_select(
 			 * parent or not
 			 */
 	assoc_mgr_lock_t locks = { READ_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+				   READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 
 	/* check to see if we are enforcing associations */
 	if (!accounting_enforce)
@@ -1302,7 +1349,7 @@ extern bool acct_policy_job_runnable_post_select(
 		if (qos_ptr->grp_cpu_mins != (uint64_t)INFINITE) {
 			if (usage_mins >= qos_ptr->grp_cpu_mins) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_QOS_GRP_CPU_MIN;
 				debug2("Job %u being held, "
 				       "the job is at or exceeds QOS %s's "
 				       "group max cpu minutes of %"PRIu64" "
@@ -1324,7 +1371,7 @@ extern bool acct_policy_job_runnable_post_select(
 				 * being killed
 				 */
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_QOS_GRP_CPU_MIN;
 				debug2("Job %u being held, "
 				       "the job is at or exceeds QOS %s's "
 				       "group max cpu minutes of %"PRIu64" "
@@ -1354,7 +1401,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && qos_ptr->grp_cpus != INFINITE) {
 			if (cpu_cnt > qos_ptr->grp_cpus) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_QOS_GRP_CPU;
 				debug2("job %u is being held, "
 				       "min cpu request %u exceeds "
 				       "group max cpu limit %u for "
@@ -1370,8 +1417,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if ((qos_ptr->usage->grp_used_cpus +
 			     cpu_cnt) > qos_ptr->grp_cpus) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+				job_ptr->state_reason =	WAIT_QOS_GRP_CPU;
 				debug2("job %u being held, "
 				       "the job is at or exceeds "
 				       "group max cpu limit %u "
@@ -1391,7 +1437,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && (qos_ptr->grp_mem != INFINITE)) {
 			if (job_memory > qos_ptr->grp_mem) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_QOS_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_QOS_GRP_MEMORY;
 				info("job %u is being held, "
 				     "memory request %u exceeds "
 				     "group max memory limit %u for "
@@ -1407,8 +1453,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if ((qos_ptr->usage->grp_used_mem +
 			     job_memory) > qos_ptr->grp_mem) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+				job_ptr->state_reason =	WAIT_QOS_GRP_MEMORY;
 				debug2("job %u being held, "
 				       "the job is at or exceeds "
 				       "group memory limit %u "
@@ -1431,7 +1476,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    qos_ptr->grp_cpu_run_mins) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_GRP_CPU_RUN_MIN;
 				debug2("job %u being held, "
 				       "qos %s is at or exceeds "
 				       "group max running cpu minutes "
@@ -1452,8 +1497,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && qos_ptr->grp_nodes != INFINITE) {
 			if (node_cnt > qos_ptr->grp_nodes) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_QOS_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_QOS_GRP_NODES;
 				debug2("job %u is being held, "
 				       "min node request %u exceeds "
 				       "group max node limit %u for "
@@ -1470,8 +1514,7 @@ extern bool acct_policy_job_runnable_post_select(
 			     node_cnt) >
 			    qos_ptr->grp_nodes) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+				job_ptr->state_reason =	WAIT_QOS_GRP_NODES;
 				debug2("job %u being held, "
 				       "the job is at or exceeds "
 				       "group max node limit %u "
@@ -1497,7 +1540,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    (job_cpu_time_limit > cpu_time_limit)) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_JOB_LIMIT;
+					WAIT_QOS_MAX_CPU_MINS_PER_JOB;
 				debug2("job %u being held, "
 				       "cpu time limit %"PRIu64" exceeds "
 				       "qos max per-job %"PRIu64"",
@@ -1514,7 +1557,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if (cpu_cnt > qos_ptr->max_cpus_pj) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_JOB_LIMIT;
+					WAIT_QOS_MAX_CPUS_PER_JOB;
 				debug2("job %u being held, "
 				       "min cpu limit %u exceeds "
 				       "qos per-job max %u",
@@ -1526,6 +1569,22 @@ extern bool acct_policy_job_runnable_post_select(
 			}
 		}
 
+		if ((job_ptr->limit_set_min_cpus != ADMIN_SET_LIMIT)
+		    && qos_ptr->min_cpus_pj != INFINITE) {
+			if (cpu_cnt && cpu_cnt < qos_ptr->min_cpus_pj) {
+				xfree(job_ptr->state_desc);
+				job_ptr->state_reason =	WAIT_QOS_MIN_CPUS;
+				debug2("%s job %u being held, "
+				       "min cpu limit %u below "
+				       "qos per-job min %u",
+				       __func__, job_ptr->job_id,
+				       cpu_cnt,
+				       qos_ptr->min_cpus_pj);
+				rc = false;
+				goto end_it;
+			}
+		}
+
 		if ((job_ptr->limit_set_min_cpus != ADMIN_SET_LIMIT) &&
 		    (qos_ptr->max_cpus_pu != INFINITE)) {
 			/* Hold the job if it exceeds the per-user
@@ -1534,7 +1593,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if (cpu_cnt > qos_ptr->max_cpus_pu) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_MAX_CPU_PER_USER;
 				debug2("job %u being held, "
 				       "min cpu limit %u exceeds "
 				       "qos per-user max %u",
@@ -1551,7 +1610,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    > qos_ptr->max_cpus_pu) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_MAX_CPU_PER_USER;
 				debug2("job %u being held, "
 				       "the user is at or would exceed "
 				       "max cpus per-user limit "
@@ -1574,7 +1633,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    qos_ptr->max_nodes_pj) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_JOB_LIMIT;
+					WAIT_QOS_MAX_NODE_PER_JOB;
 				debug2("job %u being held, "
 				       "min node limit %u exceeds "
 				       "qos max %u",
@@ -1594,7 +1653,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if (node_cnt > qos_ptr->max_nodes_pu) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_MAX_NODE_PER_USER;
 				debug2("job %u being held, "
 				       "min node per-puser limit %u exceeds "
 				       "qos max %u",
@@ -1614,7 +1673,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    > qos_ptr->max_nodes_pu) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_QOS_RESOURCE_LIMIT;
+					WAIT_QOS_MAX_NODE_PER_USER;
 				debug2("job %u being held, "
 				       "the user is at or would exceed "
 				       "max nodes per-user "
@@ -1652,7 +1711,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && (assoc_ptr->grp_cpu_mins != (uint64_t)INFINITE)) {
 			if (usage_mins >= assoc_ptr->grp_cpu_mins) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_CPU_MIN;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group max cpu minutes limit %"PRIu64" "
@@ -1675,7 +1734,7 @@ extern bool acct_policy_job_runnable_post_select(
 				 * being killed
 				 */
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_CPU_MIN;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group max cpu minutes of %"PRIu64" "
@@ -1699,8 +1758,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && (assoc_ptr->grp_cpus != INFINITE)) {
 			if (cpu_cnt > assoc_ptr->grp_cpus) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_CPU;
 				debug2("job %u being held, "
 				       "min cpu request %u exceeds "
 				       "group max cpu limit %u for "
@@ -1713,12 +1771,10 @@ extern bool acct_policy_job_runnable_post_select(
 				goto end_it;
 			}
 
-			if ((assoc_ptr->usage->grp_used_cpus +
-				    cpu_cnt) >
-				   assoc_ptr->grp_cpus) {
+			if ((assoc_ptr->usage->grp_used_cpus + cpu_cnt) >
+			    assoc_ptr->grp_cpus) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_CPU;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group max cpu limit %u "
@@ -1739,8 +1795,7 @@ extern bool acct_policy_job_runnable_post_select(
 		    && (assoc_ptr->grp_mem != INFINITE)) {
 			if (job_memory > assoc_ptr->grp_mem) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_MEMORY;
 				info("job %u being held, "
 				     "memory request %u exceeds "
 				     "group memory limit %u for "
@@ -1756,8 +1811,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if ((assoc_ptr->usage->grp_used_mem + job_memory) >
 			    assoc_ptr->grp_mem) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_MEMORY;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group memory limit %u "
@@ -1782,7 +1836,7 @@ extern bool acct_policy_job_runnable_post_select(
 			    assoc_ptr->grp_cpu_run_mins) {
 				xfree(job_ptr->state_desc);
 				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+					WAIT_ASSOC_GRP_CPU_RUN_MIN;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group max running cpu minutes "
@@ -1806,8 +1860,7 @@ extern bool acct_policy_job_runnable_post_select(
 			if (node_cnt >
 			    assoc_ptr->grp_nodes) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_NODES;
 				debug2("job %u being held, "
 				       "min node request %u exceeds "
 				       "group max node limit %u for "
@@ -1824,8 +1877,7 @@ extern bool acct_policy_job_runnable_post_select(
 			     node_cnt) >
 			    assoc_ptr->grp_nodes) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason =
-					WAIT_ASSOC_RESOURCE_LIMIT;
+				job_ptr->state_reason = WAIT_ASSOC_GRP_NODES;
 				debug2("job %u being held, "
 				       "assoc %u is at or exceeds "
 				       "group max node limit %u "
@@ -1862,7 +1914,8 @@ extern bool acct_policy_job_runnable_post_select(
 			if ((job_ptr->time_limit != NO_VAL) &&
 			    (job_cpu_time_limit > cpu_time_limit)) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason =
+					WAIT_ASSOC_MAX_CPU_MINS_PER_JOB;
 				debug2("job %u being held, "
 				       "cpu time limit %"PRIu64" exceeds "
 				       "assoc max per job %"PRIu64"",
@@ -1880,7 +1933,8 @@ extern bool acct_policy_job_runnable_post_select(
 			if (cpu_cnt >
 			    assoc_ptr->max_cpus_pj) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason =
+					WAIT_ASSOC_MAX_CPUS_PER_JOB;
 				debug2("job %u being held, "
 				       "min cpu limit %u exceeds "
 				       "account max %u",
@@ -1899,7 +1953,8 @@ extern bool acct_policy_job_runnable_post_select(
 		    && (assoc_ptr->max_nodes_pj != INFINITE)) {
 			if (node_cnt > assoc_ptr->max_nodes_pj) {
 				xfree(job_ptr->state_desc);
-				job_ptr->state_reason = WAIT_ASSOC_JOB_LIMIT;
+				job_ptr->state_reason =
+					WAIT_ASSOC_MAX_NODE_PER_JOB;
 				debug2("job %u being held, "
 				       "min node limit %u exceeds "
 				       "account max %u",
@@ -1932,7 +1987,7 @@ extern uint32_t acct_policy_get_max_nodes(struct job_record *job_ptr,
 {
 	uint32_t max_nodes_limit = INFINITE, qos_max_p_limit = INFINITE;
 	assoc_mgr_lock_t locks = { READ_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+				   READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 	slurmdb_qos_rec_t *qos_ptr = job_ptr->qos_ptr;
 	slurmdb_association_rec_t *assoc_ptr = job_ptr->assoc_ptr;
 	bool parent = 0; /* flag to tell us if we are looking at the
@@ -1948,12 +2003,20 @@ extern uint32_t acct_policy_get_max_nodes(struct job_record *job_ptr,
 
 	assoc_mgr_lock(&locks);
 	if (qos_ptr) {
-		qos_max_p_limit = max_nodes_limit =
-			MIN(qos_ptr->max_nodes_pj, qos_ptr->max_nodes_pu);
-		max_nodes_limit =
-			MIN(max_nodes_limit, qos_ptr->grp_nodes);
-		if (max_nodes_limit != INFINITE)
-			*wait_reason = WAIT_QOS_JOB_LIMIT;
+		if (qos_ptr->max_nodes_pj < qos_ptr->max_nodes_pu) {
+			max_nodes_limit = qos_ptr->max_nodes_pj;
+			*wait_reason = WAIT_QOS_MAX_NODE_PER_JOB;
+		} else if (qos_ptr->max_nodes_pu != INFINITE) {
+			max_nodes_limit = qos_ptr->max_nodes_pu;
+			*wait_reason = WAIT_QOS_MAX_NODE_PER_USER;
+		}
+
+		qos_max_p_limit = max_nodes_limit;
+
+		if (qos_ptr->grp_nodes < max_nodes_limit) {
+			max_nodes_limit = qos_ptr->grp_nodes;
+			*wait_reason = WAIT_QOS_GRP_NODES;
+		}
 	}
 
 	/* We have to traverse all the associations because QOS might
@@ -1964,7 +2027,7 @@ extern uint32_t acct_policy_get_max_nodes(struct job_record *job_ptr,
 		    && (assoc_ptr->grp_nodes != INFINITE)
 		    && (assoc_ptr->grp_nodes < max_nodes_limit)) {
 			max_nodes_limit = assoc_ptr->grp_nodes;
-			*wait_reason = WAIT_ASSOC_JOB_LIMIT;
+			*wait_reason = WAIT_ASSOC_GRP_NODES;
 			grp_set = 1;
 		}
 
@@ -1973,7 +2036,7 @@ extern uint32_t acct_policy_get_max_nodes(struct job_record *job_ptr,
 		    && (assoc_ptr->max_nodes_pj != INFINITE)
 		    && (assoc_ptr->max_nodes_pj < max_nodes_limit)) {
 			max_nodes_limit = assoc_ptr->max_nodes_pj;
-			*wait_reason = WAIT_ASSOC_JOB_LIMIT;
+			*wait_reason = WAIT_ASSOC_MAX_NODE_PER_JOB;
 		}
 
 		/* only check the first grp set */
@@ -2132,7 +2195,7 @@ extern bool acct_policy_job_time_out(struct job_record *job_ptr)
 	slurmdb_qos_rec_t *qos = NULL;
 	slurmdb_association_rec_t *assoc = NULL;
 	assoc_mgr_lock_t locks = { READ_LOCK, NO_LOCK,
-				   READ_LOCK, NO_LOCK, NO_LOCK };
+				   READ_LOCK, NO_LOCK, NO_LOCK, NO_LOCK };
 	time_t now;
 
 	/* Now see if we are enforcing limits.  If Safe is set then
