@@ -92,9 +92,12 @@ _valid_uid_gid(uid_t uid, gid_t *gid, char **user_name)
 	if (*user_name)
 		return 1;
 #endif
-
+znovu:
+	errno = 0;
 	pwd = getpwuid(uid);
 	if (!pwd) {
+		if (errno == EINTR)
+			goto znovu;
 		error("uid %ld not found on system", (long) uid);
 		slurm_seterrno(ESLURMD_UID_NOT_FOUND);
 		return 0;
@@ -423,10 +426,16 @@ stepd_step_rec_create(launch_tasks_request_msg_t *msg)
 	format_core_allocs(msg->cred, conf->node_name, conf->cpus,
 			   &job->job_alloc_cores, &job->step_alloc_cores,
 			   &job->job_mem, &job->step_mem);
-	if (job->step_mem) {
+
+	/* If users have configured MemLimitEnforce=no
+	 * in their slurm.conf keep going.
+	 */
+	if (job->step_mem
+	    && conf->mem_limit_enforce) {
 		jobacct_gather_set_mem_limit(job->jobid, job->stepid,
 					     job->step_mem);
-	} else if (job->job_mem) {
+	} else if (job->job_mem
+		   && conf->mem_limit_enforce) {
 		jobacct_gather_set_mem_limit(job->jobid, job->stepid,
 					     job->job_mem);
 	}
@@ -529,9 +538,11 @@ batch_stepd_step_rec_create(batch_job_launch_msg_t *msg)
 	format_core_allocs(msg->cred, conf->node_name, conf->cpus,
 			   &job->job_alloc_cores, &job->step_alloc_cores,
 			   &job->job_mem, &job->step_mem);
-	if (job->step_mem)
+	if (job->step_mem
+		&& conf->mem_limit_enforce)
 		jobacct_gather_set_mem_limit(job->jobid, NO_VAL, job->step_mem);
-	else if (job->job_mem)
+	else if (job->job_mem
+		&& conf->mem_limit_enforce)
 		jobacct_gather_set_mem_limit(job->jobid, NO_VAL, job->job_mem);
 
 	get_cred_gres(msg->cred, conf->node_name,
