@@ -67,10 +67,13 @@
 
 #include "src/slurmdbd/read_config.h"
 #include "src/slurmdbd/rpc_mgr.h"
+#include "src/slurmdbd/proc_req.h"
 #include "src/slurmdbd/backup.h"
 
 /* Global variables */
 time_t shutdown_time = 0;		/* when shutdown request arrived */
+List registered_clusters = NULL;
+pthread_mutex_t registered_lock = PTHREAD_MUTEX_INITIALIZER;
 
 /* Local variables */
 static int    dbd_sigarray[] = {	/* blocked signals for this process */
@@ -85,8 +88,10 @@ static int	 new_nice = 0;
 static pthread_t rpc_handler_thread;	/* thread ID for RPC hander */
 static pthread_t signal_handler_thread;	/* thread ID for signal hander */
 static pthread_t rollup_handler_thread;	/* thread ID for rollup hander */
+static pthread_t commit_handler_thread;	/* thread ID for commit hander */
 static pthread_mutex_t rollup_lock = PTHREAD_MUTEX_INITIALIZER;
 static bool running_rollup = 0;
+static bool running_commit = 0;
 
 /* Local functions */
 static void  _become_slurm_user(void);
@@ -99,6 +104,8 @@ static void  _parse_commandline(int argc, char *argv[]);
 static void  _request_registrations(void *db_conn);
 static void  _rollup_handler_cancel();
 static void *_rollup_handler(void *no_data);
+static void  _commit_handler_cancel();
+static void *_commit_handler(void *no_data);
 static int   _send_slurmctld_register_req(slurmdb_cluster_rec_t *cluster_rec);
 static void  _set_work_dir(void);
 static void *_signal_handler(void *no_data);
@@ -163,6 +170,14 @@ int main(int argc, char *argv[])
 		fatal("pthread_create %m");
 	slurm_attr_destroy(&thread_attr);
 
+	registered_clusters = list_create(NULL);
+
+	slurm_attr_init(&thread_attr);
+	if (pthread_create(&commit_handler_thread, &thread_attr,
+			   _commit_handler, NULL))
+		fatal("pthread_create %m");
+	slurm_attr_destroy(&thread_attr);
+
 	memset(&assoc_init_arg, 0, sizeof(assoc_init_args_t));
 
 	/* If we are tacking wckey we need to cache
@@ -171,7 +186,7 @@ int main(int argc, char *argv[])
 	if (slurmdbd_conf->track_wckey)
 		assoc_init_arg.cache_level |= ASSOC_MGR_CACHE_WCKEY;
 
-	db_conn = acct_storage_g_get_connection(NULL, 0, false, NULL);
+	db_conn = acct_storage_g_get_connection(NULL, 0, true, NULL);
 	if (assoc_mgr_init(db_conn, &assoc_init_arg, errno) == SLURM_ERROR) {
 		error("Problem getting cache of data");
 		acct_storage_g_close_connection(&db_conn);
@@ -188,6 +203,8 @@ int main(int argc, char *argv[])
 			info("slurmdbd running in background mode");
 			have_control = false;
 			backup = true;
+			/* make sure any locks are released */
+			acct_storage_g_commit(db_conn, 1);
 			run_dbd_backup();
 			if (!shutdown_time)
 				assoc_mgr_refresh_lists(db_conn);
@@ -232,6 +249,7 @@ int main(int argc, char *argv[])
 		}
 
 		_request_registrations(db_conn);
+		acct_storage_g_commit(db_conn, 1);
 
 		/* this is only ran if not backup */
 		if (rollup_handler_thread)
@@ -249,10 +267,14 @@ int main(int argc, char *argv[])
 	}
 	/* Daemon termination handled here */
 
+end_it:
+
 	if (signal_handler_thread)
 		pthread_join(signal_handler_thread, NULL);
+	if (commit_handler_thread)
+		pthread_join(commit_handler_thread, NULL);
 
-end_it:
+	acct_storage_g_commit(db_conn, 1);
 	acct_storage_g_close_connection(&db_conn);
 
 	if (slurmdbd_conf->pid_file &&
@@ -261,6 +283,8 @@ end_it:
 			slurmdbd_conf->pid_file);
 	}
 
+	FREE_NULL_LIST(registered_clusters);
+
 	assoc_mgr_fini(NULL);
 	slurm_acct_storage_fini();
 	slurm_auth_fini();
@@ -269,9 +293,21 @@ end_it:
 	exit(0);
 }
 
+extern void reconfig()
+{
+	read_slurmdbd_conf();
+	assoc_mgr_set_missing_uids();
+	acct_storage_g_reconfig(NULL, 0);
+	_update_logging(false);
+}
+
 extern void shutdown_threads()
 {
 	shutdown_time = time(NULL);
+	/* End commit before rpc_mgr_wake.  It will do the final
+	   commit on the connection.
+	*/
+	_commit_handler_cancel();
 	rpc_mgr_wake();
 	_rollup_handler_cancel();
 }
@@ -552,6 +588,7 @@ static void *_rollup_handler(void *db_conn)
 		running_rollup = 1;
 		debug2("running rollup at %s", slurm_ctime(&start_time));
 		acct_storage_g_roll_usage(db_conn, 0, 0, 1);
+		acct_storage_g_commit(db_conn, 1);
 		running_rollup = 0;
 		slurm_mutex_unlock(&rollup_lock);
 
@@ -583,6 +620,52 @@ static void *_rollup_handler(void *db_conn)
 	return NULL;
 }
 
+static void _commit_handler_cancel()
+{
+	if (running_commit)
+		debug("Waiting for commit thread to finish.");
+	slurm_mutex_lock(&registered_lock);
+	if (commit_handler_thread)
+		pthread_cancel(commit_handler_thread);
+	slurm_mutex_unlock(&registered_lock);
+}
+
+/* _commit_handler - Process commit's of registered clusters */
+static void *_commit_handler(void *db_conn)
+{
+	ListIterator itr;
+	slurmdbd_conn_t *slurmdbd_conn;
+
+	(void) pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
+	(void) pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+
+	while (!shutdown_time) {
+		/* Commit each slurmctld's info */
+		if (slurmdbd_conf->commit_delay) {
+			slurm_mutex_lock(&registered_lock);
+			running_commit = 1;
+			itr = list_iterator_create(registered_clusters);
+			while ((slurmdbd_conn = list_next(itr))) {
+				debug4("running commit for %s",
+				       slurmdbd_conn->cluster_name);
+				acct_storage_g_commit(
+					slurmdbd_conn->db_conn, 1);
+			}
+			list_iterator_destroy(itr);
+			running_commit = 0;
+			slurm_mutex_unlock(&registered_lock);
+		}
+
+		/* This really doesn't need to be synconized so just
+		 * sleep for a bit and do it again.
+		 */
+		sleep(slurmdbd_conf->commit_delay ?
+		      slurmdbd_conf->commit_delay : 5);
+	}
+
+	return NULL;
+}
+
 /*
  * send_slurmctld_register_req - request register from slurmctld
  * IN host: control host of cluster
@@ -642,9 +725,7 @@ static void *_signal_handler(void *no_data)
 		switch (sig) {
 		case SIGHUP:	/* kill -1 */
 			info("Reconfigure signal (SIGHUP) received");
-			read_slurmdbd_conf();
-			assoc_mgr_set_missing_uids();
-			_update_logging(false);
+			reconfig();
 			break;
 		case SIGINT:	/* kill -2  or <CTRL-C> */
 		case SIGTERM:	/* kill -15 */
